(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    270257,       8725]
NotebookOptionsPosition[    219635,       7338]
NotebookOutlinePosition[    241524,       7815]
CellTagsIndexPosition[    240518,       7782]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Modeling Computation", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 13}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 "In this chapter we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to study theoretical models of computation. We will see how to generate \
elements of a language from a type 2 phrase-structure grammar and how to \
implement finite-state machines with and without output. We will also examine \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s support for regular expressions and implement Turing machines."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["13.1 Languages and Grammars", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "We will write a function to generate elements of a language from a type 2 \
phrase-structure grammar. Recall that a type 2 grammar has productions only \
of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["w", "1"], "\[Rule]", 
    SubscriptBox["w", "2"]}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["w", "1"], TraditionalForm]]],
 " a single nonterminal symbol."
}], "Text"],

Cell[TextData[{
 "Our strategy for generating the language will be as follows. We initialize \
a list ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " to the empty list. In this list, we will store all words, that is, strings \
consisting only of terminal symbols. A list ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is initialized to the list consisting of the starting symbol."
}], "Text"],

Cell[TextData[{
 "We process an element of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " by removing it from the list and applying all possible productions to it. \
The results of the productions are either added to ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " if they consist solely of terminal symbols, or placed in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to be processed further."
}], "Text"],

Cell[TextData[{
 "In order to prevent the time taken from becoming excessive, we will impose \
a time limit using the ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " function. This limit will be an argument to the function."
}], "Text"],

Cell[CellGroupData[{

Cell["Representation", "Subsection",
 CellTags->"Section1a"],

Cell[TextData[{
 "We first need to determine how we will model the elements of the grammar in \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell["\<\
We will generally represent terminal symbols as lower case letters stored as \
characters (strings). Nonterminal symbols will be upper case letters, also \
entered as strings. \
\>", "Text"],

Cell["\<\
Strings containing nonterminal symbols and words will be stored as strings. \
Productions will be stored in an indexed variable. The indices will be the \
nonterminal symbols (recall that we're considering only type 2 grammars). The \
value associated to a nonterminal symbol will be the list of all products \
derivable from that symbol.\
\>", "Text"],

Cell[TextData[{
 "In Example 12 in the textbook, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", "\[Rule]", "AB"}], TraditionalForm]]],
 " is the only derivation from the starting symbol, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "\"\<AB\>\"", "}"}], TraditionalForm]]],
 " will be the entry associated to ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " in the indexed variable. On the other hand, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", "\[Rule]", "Ba"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", "\[Rule]", "Cb"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", "\[Rule]", "b"}], TraditionalForm]]],
 " are all productions from ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<Ba\>\"", ",", "\"\<Bc\>\"", ",", "\"\<b\>\""}], "}"}], 
   TraditionalForm]]],
 " would be the entry associated to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 "."
}], "Text"],

Cell["Here are the productions for Example 12.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ex12productions", "[", "\"\<S\>\"", "]"}], "=", 
   RowBox[{"{", "\"\<AB\>\"", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex12productions", "[", "\"\<A\>\"", "]"}], "=", 
   RowBox[{"{", "\"\<Ca\>\"", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex12productions", "[", "\"\<B\>\"", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Ba\>\"", ",", "\"\<Cb\>\"", ",", "\"\<b\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex12productions", "[", "\"\<C\>\"", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"\"\<cb\>\"", ",", "\"\<b\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"?", "ex12productions"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["Global`ex12productions", "Print", "PrintUsage",
 CellTags->"Info3580720243-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"ex12productions", "[", "\<\"A\"\>", "]"}], "=", 
         RowBox[{"{", "\<\"Ca\"\>", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex12productions", "[", "\<\"B\"\>", "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"\<\"Ba\"\>", ",", "\<\"Cb\"\>", ",", "\<\"b\"\>"}], 
          "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex12productions", "[", "\<\"C\"\>", "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"\<\"cb\"\>", ",", "\<\"b\"\>"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex12productions", "[", "\<\"S\"\>", "]"}], "=", 
         RowBox[{"{", "\<\"AB\"\>", "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`ex12productions],
  Editable->False]], "Print",
 CellTags->"Info3580720243-4013843"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "Our function will require the following arguments: the set ",
 StyleBox["V", "Input"],
 " defining the vocabulary, the set ",
 StyleBox["T", "Input"],
 " of terminal symbols, the starting symbol ",
 StyleBox["S", "Input"],
 ", the table of productions ",
 StyleBox["P", "Input"],
 ", and the limit on the time, in seconds, ",
 StyleBox["timelimit", "Input"],
 ". Note that, with the exception of the time limit, this is the same \
information that makes up a grammar."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Implementation", "Subsection",
 CellTags->"Section1b"],

Cell[TextData[{
 "The function begins by initializing ",
 StyleBox["L", "Input"],
 " to the empty set and ",
 StyleBox["A", "Input"],
 " to the list containing the starting string as the sole element. Recall \
that ",
 StyleBox["L", "Input"],
 " and ",
 StyleBox["A", "Input"],
 " will store the words that have been produced and the list of strings with \
nonterminal symbols that still require processing. "
}], "Text"],

Cell[TextData[{
 "After the initializations are complete, we begin a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop controlled by the condition that ",
 StyleBox["A", "Input"],
 " is nonempty. Within the loop, set ",
 StyleBox["curString", "Input"],
 " (the \[OpenCurlyDoubleQuote]current string\[CloseCurlyDoubleQuote]) equal \
to the first element of ",
 StyleBox["A", "Input"],
 " and remove it from ",
 StyleBox["A", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "We need to find all the strings that are directly derivable from ",
 StyleBox["curString", "Input"],
 ". We do this as follows. First, determine the nonterminal symbols ",
 StyleBox["N", "Input"],
 " by computing the complement of the terminal symbols ",
 StyleBox["T", "Input"],
 " relative to the vocabulary ",
 StyleBox["V", "Input"],
 ". Also initialize a list ",
 StyleBox["D", "Input"],
 " (for derivations) to the empty list. We will store all the strings derived \
from ",
 StyleBox["curString", "Input"],
 " in this list and then later determine which should be added to ",
 StyleBox["L", "Input"],
 " and which to ",
 StyleBox["A", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Remember that ",
 StyleBox["curString", "Input"],
 " is represented as a string. We can use ",
 StyleBox[ButtonBox["StringPosition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringPosition"], "FunctionLink"],
 " to determine whether a particular nonterminal ",
 StyleBox["symbol", "CodeVar"],
 " appears in a ",
 StyleBox["string", "CodeVar"],
 " by evaluating the expression ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"StringPosition", "["}], 
   StyleBox["string", "CodeVar"], ",", 
   StyleBox["symbol", "CodeVar"], "]"}]], "myCode"],
 ". The output will be a list of lists with each inner list specifying the \
location of an occurance of ",
 StyleBox["symbol", "CodeVar"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringPosition", "[", 
  RowBox[{"\"\<AbcAb\>\"", ",", "\"\<A\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "4"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the positions for the string \[OpenCurlyDoubleQuote]A\
\[CloseCurlyDoubleQuote] are given as ranges. This is because ",
 StyleBox[ButtonBox["StringPosition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringPosition"], "FunctionLink"],
 " is often used to find substrings of more than one character, so the \
function is returning a list of ranges."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringPosition", "[", 
  RowBox[{"\"\<abccbabca\>\"", ",", "\"\<ab\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "7"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that the result when the target string is not found is the empty list.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringPosition", "[", 
  RowBox[{"\"\<AbcAb\>\"", ",", "\"\<X\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "For a given ",
 StyleBox["curString", "Input"],
 ", we will loop over the nonterminal symbols. For any nonterminal symbols \
that are found, we look the symbol up in the production table ",
 StyleBox["P", "Input"],
 ". For each associated production, we perform a substitution."
}], "Text"],

Cell["\<\
An example may be helpful to explain this step. Suppose we are processing the \
string \[OpenCurlyDoubleQuote]cBbaBa\[CloseCurlyDoubleQuote] as part of the \
grammar given in Example 12 of Section 13.1.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"curString", "=", "\"\<cBbaBa\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"cBbaBa\"\>"], "Output"]
}, Open  ]],

Cell["\<\
First we check for the nonterminal symbol \[OpenCurlyDoubleQuote]A\
\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringPosition", "[", 
  RowBox[{"curString", ",", "\"\<A\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Since \[OpenCurlyDoubleQuote]A\[CloseCurlyDoubleQuote] is not present, we \
move on to \[OpenCurlyDoubleQuote]B\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringPosition", "[", 
  RowBox[{"curString", ",", "\"\<B\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "5"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
We see that \[OpenCurlyDoubleQuote]B\[CloseCurlyDoubleQuote] does occur in \
the string. So we look up \[OpenCurlyDoubleQuote]B\[CloseCurlyDoubleQuote] in \
the production table.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex12productions", "[", "\"\<B\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Ba\"\>", ",", "\<\"Cb\"\>", ",", "\<\"b\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We have two occurances of the nonterminal symbol \[OpenCurlyDoubleQuote]B\
\[CloseCurlyDoubleQuote] and three productions. Applying each production to \
each location will produce six new strings, each of which has one of the \
occurances of \[OpenCurlyDoubleQuote]B\[CloseCurlyDoubleQuote] replaced. We \
use a Do loop with two loop specifications: one over the productions and one \
over the list of positions. We\[CloseCurlyQuote]ll be using the ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["variable", "CodeVar"], ",", 
   StyleBox["list", "CodeVar"], "}"}]], "myCode"],
 " form of the loop specifications. Note that if ",
 StyleBox["list", "CodeVar"],
 " is the empty list, then no iteration will occur. "
}], "Text"],

Cell[TextData[{
 "We apply the derivation with the ",
 StyleBox[ButtonBox["StringReplacePart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringReplacePart"], "FunctionLink"],
 " function. This function requires three arguments. The first is the \
original string, in this case ",
 StyleBox["curString", "Input"],
 ". The second argument is the new string, in this case the element from the \
list of productions. And the third argument is the location being replaced, \
in the same format as output from ",
 StyleBox[ButtonBox["StringPosition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringPosition"], "FunctionLink"],
 ". For example, below we replace \[OpenCurlyDoubleQuote]xyz\
\[CloseCurlyDoubleQuote] with \
\[OpenCurlyDoubleQuote]d\[CloseCurlyDoubleQuote]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringReplacePart", "[", 
  RowBox[{"\"\<abcxyzefg\>\"", ",", "\"\<d\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "6"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["\<\"abcdefg\"\>"], "Output"]
}, Open  ]],

Cell["These elements combine to the following code.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"Print", "[", 
    RowBox[{"StringReplacePart", "[", 
     RowBox[{"curString", ",", "p", ",", "l"}], "]"}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"p", ",", 
     RowBox[{"ex12productions", "[", "\"\<B\>\"", "]"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"l", ",", 
     RowBox[{"StringPosition", "[", 
      RowBox[{"curString", ",", "\"\<B\>\""}], "]"}]}], "}"}]}], 
  "\[IndentingNewLine]", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData["\<\"cBabaBa\"\>"], "Print"],

Cell[BoxData["\<\"cBbaBaa\"\>"], "Print"],

Cell[BoxData["\<\"cCbbaBa\"\>"], "Print"],

Cell[BoxData["\<\"cBbaCba\"\>"], "Print"],

Cell[BoxData["\<\"cbbaBa\"\>"], "Print"],

Cell[BoxData["\<\"cBbaba\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "In our function, instead of printing the productions, we will ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " them and enclose the loop in a ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 ". We will also enclose the loop illustrated above within another ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop over all of the nonterminal symbols. The resulting list of derived \
strings is stored as ",
 StyleBox["D", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Once ",
 StyleBox["curString", "Input"],
 " has been completely processed, we turn to deciding whether each element we \
placed in ",
 StyleBox["D", "Input"],
 " is a word or not. The most straightforward way to approach this is to \
consider whether or not it contains any nonterminal symbols. We can do this \
by using ",
 StyleBox[ButtonBox["StringPosition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringPosition"], "FunctionLink"],
 " again, this time with the list of all nonterminal symbols as the second \
argument. With a list as the second argument, ",
 StyleBox[ButtonBox["StringPosition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringPosition"], "FunctionLink"],
 " outputs the list of all matches for any members of the list. If the output \
is the empty list, that tells us that the string has no nonterminal symbols, \
and is thus a word."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringPosition", "[", 
  RowBox[{"\"\<babaaa\>\"", ",", 
   RowBox[{"{", 
    RowBox[{
    "\"\<S\>\"", ",", "\"\<A\>\"", ",", "\"\<B\>\"", ",", "\"\<C\>\""}], 
    "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell["Here is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"formWords", "[", 
   RowBox[{"V_", ",", "T_", ",", "S_", ",", "P_", ",", "timelimit_"}], "]"}], 
  ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"L", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"A", "=", 
       RowBox[{"{", "\"\<S\>\"", "}"}]}], ",", "N", ",", "curString", ",", 
      "D", ",", "s", ",", "d"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"N", "=", 
      RowBox[{"Complement", "[", 
       RowBox[{"V", ",", "T"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"TimeConstrained", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{"A", "\[NotEqual]", 
          RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"curString", "=", 
           RowBox[{"A", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"A", "=", 
           RowBox[{"Delete", "[", 
            RowBox[{"A", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"D", "=", 
           RowBox[{
            RowBox[{"Reap", "[", "\[IndentingNewLine]", 
             RowBox[{"Do", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Do", "[", 
                RowBox[{
                 RowBox[{"Sow", "[", 
                  RowBox[{"StringReplacePart", "[", 
                   RowBox[{"curString", ",", "p", ",", "l"}], "]"}], "]"}], 
                 ",", "\[IndentingNewLine]", 
                 RowBox[{"{", 
                  RowBox[{"p", ",", 
                   RowBox[{"P", "[", "s", "]"}]}], "}"}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"{", 
                  RowBox[{"l", ",", 
                   RowBox[{"StringPosition", "[", 
                    RowBox[{"curString", ",", "s"}], "]"}]}], "}"}]}], 
                "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", ",", 
               RowBox[{"{", 
                RowBox[{"s", ",", "N"}], "}"}]}], "]"}], "]"}], "[", 
            RowBox[{"[", 
             RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Do", "[", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"StringPosition", "[", 
                RowBox[{"d", ",", "N"}], "]"}], "\[Equal]", 
               RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"L", ",", "d"}], "]"}], ",", "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"A", ",", "d"}], "]"}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"d", ",", "D"}], "}"}]}], "\[IndentingNewLine]", 
           "]"}]}]}], "\[IndentingNewLine]", "]"}], ",", "timelimit"}], "]"}],
      ";", "\[IndentingNewLine]", 
     RowBox[{"DeleteDuplicates", "[", "L", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We use our function on the grammar defined by Example 12, up to one tenth of \
a second.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"formWords", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<A\>\"", ",", 
     "\"\<B\>\"", ",", "\"\<C\>\"", ",", "\"\<S\>\""}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], ",", 
   "\"\<S\>\"", ",", "ex12productions", ",", ".1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"cbab\"\>", ",", "\<\"bab\"\>", ",", "\<\"cbaba\"\>", 
   ",", "\<\"baba\"\>", ",", "\<\"cbacbb\"\>", ",", "\<\"cbabb\"\>", 
   ",", "\<\"bacbb\"\>", ",", "\<\"babb\"\>", ",", "\<\"cbabaa\"\>", 
   ",", "\<\"babaa\"\>", ",", "\<\"cbacbba\"\>", ",", "\<\"cbabba\"\>", 
   ",", "\<\"bacbba\"\>", ",", "\<\"babba\"\>", ",", "\<\"cbabaaa\"\>", 
   ",", "\<\"babaaa\"\>", ",", "\<\"cbacbbaa\"\>", ",", "\<\"cbabbaa\"\>", 
   ",", "\<\"bacbbaa\"\>", ",", "\<\"babbaa\"\>", ",", "\<\"cbabaaaa\"\>", 
   ",", "\<\"babaaaa\"\>", ",", "\<\"cbacbbaaa\"\>", ",", "\<\"cbabbaaa\"\>", 
   ",", "\<\"bacbbaaa\"\>", ",", "\<\"babbaaa\"\>", ",", "\<\"cbabaaaaa\"\>", 
   ",", "\<\"babaaaaa\"\>", ",", "\<\"cbacbbaaaa\"\>", 
   ",", "\<\"cbabbaaaa\"\>", ",", "\<\"bacbbaaaa\"\>", 
   ",", "\<\"babbaaaa\"\>", ",", "\<\"cbabaaaaaa\"\>", 
   ",", "\<\"babaaaaaa\"\>", ",", "\<\"cbacbbaaaaa\"\>", 
   ",", "\<\"cbabbaaaaa\"\>", ",", "\<\"bacbbaaaaa\"\>", 
   ",", "\<\"babbaaaaa\"\>", ",", "\<\"cbabaaaaaaa\"\>", 
   ",", "\<\"babaaaaaaa\"\>", ",", "\<\"cbacbbaaaaaa\"\>", 
   ",", "\<\"cbabbaaaaaa\"\>", ",", "\<\"bacbbaaaaaa\"\>", 
   ",", "\<\"babbaaaaaa\"\>", ",", "\<\"cbabaaaaaaaa\"\>", 
   ",", "\<\"babaaaaaaaa\"\>", ",", "\<\"cbacbbaaaaaaa\"\>", 
   ",", "\<\"cbabbaaaaaaa\"\>", ",", "\<\"bacbbaaaaaaa\"\>", 
   ",", "\<\"babbaaaaaaa\"\>", ",", "\<\"cbabaaaaaaaaa\"\>", 
   ",", "\<\"babaaaaaaaaa\"\>", ",", "\<\"cbacbbaaaaaaaa\"\>", 
   ",", "\<\"cbabbaaaaaaaa\"\>", ",", "\<\"bacbbaaaaaaaa\"\>", 
   ",", "\<\"babbaaaaaaaa\"\>", ",", "\<\"cbabaaaaaaaaaa\"\>", 
   ",", "\<\"babaaaaaaaaaa\"\>", ",", "\<\"cbacbbaaaaaaaaa\"\>", 
   ",", "\<\"cbabbaaaaaaaaa\"\>", ",", "\<\"bacbbaaaaaaaaa\"\>", 
   ",", "\<\"babbaaaaaaaaa\"\>", ",", "\<\"cbabaaaaaaaaaaa\"\>", 
   ",", "\<\"babaaaaaaaaaaa\"\>", ",", "\<\"cbacbbaaaaaaaaaa\"\>", 
   ",", "\<\"cbabbaaaaaaaaaa\"\>", ",", "\<\"bacbbaaaaaaaaaa\"\>", 
   ",", "\<\"babbaaaaaaaaaa\"\>"}], "}"}]], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["13.2 Finite-State Machines with Output", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 "Example 4 from Section 13.2 describes a finite-state machine with five \
states and with input and output alphabets both equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], TraditionalForm]]],
 ". Example 6 describes how to implement addition of integers using their \
binary expressions with a finite-state machine with output. Here, we will use \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to model those two finite-state machines. We will model strings in the \
language as lists."
}], "Text"],

Cell[CellGroupData[{

Cell["A First Example", "Subsection",
 CellTags->"Section2a"],

Cell[TextData[{
 "Recall from Definition 1 in Section 13.2 that a finite-state machine \
consists of six objects: a set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " of states, an input alphabet ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", an output alphabet ",
 Cell[BoxData[
  FormBox["O", TraditionalForm]]],
 ", a transition function ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 ", an output function ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ", and an initial state ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["s", "0"], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We will write a function that, given that information and an input string, \
will return the associated output string. Specifically, we will give as an \
argument to the function a list of members of the input alphabet, and the \
function will return a list of members of the output alphabet such that the \
",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th element in the output list is the output associated with the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th member of the input list."
}], "Text"],

Cell[CellGroupData[{

Cell["Representation", "Subsubsection"],

Cell[TextData[{
 "As is typical, we must first describe how we will represent the necessary \
objects in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[TextData[{
 "The states will be represented by nonnegative integers. For example, in \
Example 4, the states will be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "2", ",", "3", ",", "4"}], "}"}], 
   TraditionalForm]]],
 ". We will assume, for the sake of simplicity, that the initial state will \
always be state 0. Neither ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["s", "0"], TraditionalForm]]],
 " are therefore required as arguments to the function."
}], "Text"],

Cell[TextData[{
 "The input and output alphabets, ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["O", TraditionalForm]]],
 " can be represented by lists of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " objects but will not be required arguments to the function as they can be \
inferred from the transition and output function. In Example 4, these are \
both equal to the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The transition function and output function will be represented by a single \
indexed variable. This will have the benefit of making the definition of the \
functions less cumbersome. The indices to the variable will be pairs ",
 Cell[BoxData[
  RowBox[{"[", 
   StyleBox["state", "CodeVar"], ",", 
   StyleBox["input", "CodeVar"], "]"}]], "myCode"],
 " where ",
 Cell[BoxData[
  FormBox[
   StyleBox["state", "CodeVar"], TraditionalForm]]],
 " is a nonnegative integer and ",
 Cell[BoxData[
  FormBox[
   StyleBox["input", "CodeVar"], TraditionalForm]]],
 " will be a member of ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ". The values of the variable will be pairs ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["newState", "CodeVar"], ",", 
   StyleBox["output", "CodeVar"], "}"}]], "myCode"],
 ", where ",
 Cell[BoxData[
  FormBox[
   StyleBox["newState", "CodeVar"], TraditionalForm]]],
 " is the state transitioned to and ",
 Cell[BoxData[
  FormBox[
   StyleBox["output", "CodeVar"], TraditionalForm]]],
 " is the output corresponding to the original state and the input."
}], "Text"],

Cell["\<\
Here is the definition of the transition-output table for Example 4. (Refer \
to Table 3 of Section 13.2 as the source of the values in the table.)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"0", ",", "0"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"0", ",", "1"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"3", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"1", ",", "0"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"2", ",", "0"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"3", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"4", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"3", ",", "0"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"3", ",", "1"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"4", ",", "0"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"3", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex4Table", "[", 
    RowBox[{"4", ",", "1"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"4", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"?", "ex4Table"}]}], "Input",
 GroupPageBreakWithin->True,
 InitializationCell->True],

Cell[CellGroupData[{

Cell["Global`ex4Table", "Print", "PrintUsage",
 CellTags->"Info3580720244-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"0", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"0", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "0"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"1", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"2", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "0"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"4", ",", "0"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"3", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"3", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"4", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "0"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex4Table", "[", 
          RowBox[{"4", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"4", ",", "0"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`ex4Table],
  Editable->False]], "Print",
 CellTags->"Info3580720244-4013843"]
}, Open  ]]
}, Open  ]],

Cell["\<\
Observe that the indices for the transition-output table consist of every \
possible state-input pair.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Machine Modeling Function", "Subsubsection"],

Cell["\<\
The function we create will accept as arguments the name of the indexed \
variable representing the transition-output table and the input string. It \
will produce the output string.\
\>", "Text"],

Cell[TextData[{
 "The function is fairly straightforward. Initialize the current state of the \
machine, stored in ",
 StyleBox["curState", "Input"],
 ", to 0, since we are insisting that 0 represent the starting state. Also \
initialize the output string, ",
 StyleBox["outString", "Input"],
 ", to the list of all 0s of the same length as the input list. (It is more \
efficient, when the length of a list is known in advance, to initialize it to \
the correct length than it is to build it one element at a time.)"
}], "Text"],

Cell[TextData[{
 "Begin a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop from 1 to the length of the input string. For each index, look up the \
pair consisting of ",
 StyleBox["curState", "Input"],
 " and the element in the input string in the transition-output table. The \
second element in the result is placed in the output string at the correct \
position, and the first element is used to update ",
 StyleBox["curState", "Input"],
 ". Once the loop is complete, the output list is returned."
}], "Text"],

Cell["Here is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"machineWithOutput", "[", 
   RowBox[{"transTable_", ",", "inString_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"curState", "=", "0"}], ",", "outString", ",", "i", ",", "newo",
       ",", "news"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"outString", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", 
        RowBox[{"Length", "[", "inString", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "inString", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"news", ",", "newo"}], "}"}], "=", 
         RowBox[{"transTable", "[", 
          RowBox[{"curState", ",", 
           RowBox[{"inString", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"outString", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "=", "newo"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"curState", "=", "news"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "outString"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Example 4 asks to find the output string when the input is ",
 Cell[BoxData[
  FormBox["101011", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"machineWithOutput", "[", 
  RowBox[{"ex4Table", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "1"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "1", ",", "0", ",", "0", ",", "0"}], 
  "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["A Finite-State Machine for Addition", "Subsection",
 CellTags->"Section2b"],

Cell["\<\
Example 6 in Section 13.2 describes how a finite-state machine with output \
that adds two integers using their binary expansions can be designed. Figure \
5 in the text gives a diagram illustrating the machine.\
\>", "Text"],

Cell[TextData[{
 "The input alphabet for this machine are the four bit pairs: ",
 Cell[BoxData[
  FormBox["00", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["01", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["10", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["11", TraditionalForm]]],
 ". We will represent the pairs as strings. As described by the text, we \
assume that the initial bits ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "n"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["y", "n"], TraditionalForm]]],
 " are both ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "As an example, consider adding ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", "=", 
    SubscriptBox["0111", "2"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", "=", 
    SubscriptBox["0110", "2"]}], TraditionalForm]]],
 ". We input these two numbers as pairs and in reverse order. Thus the input \
string will be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"10", ",", "11", ",", "11", ",", "00"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
The transition-output table is obtained from the diagram shown in Figure 5.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"0", ",", "\"\<00\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"0", ",", "\"\<01\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"0", ",", "\"\<10\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"0", ",", "\"\<11\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"1", ",", "\"\<00\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"1", ",", "\"\<01\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"1", ",", "\"\<10\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"addTable", "[", 
    RowBox[{"1", ",", "\"\<11\>\""}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying the ",
 StyleBox["machineWithOutput", "Input"],
 " function to this table and the input produces the sum of the integers."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"machineWithOutput", "[", 
  RowBox[{"addTable", ",", 
   RowBox[{"{", 
    RowBox[{
    "\"\<10\>\"", ",", "\"\<11\>\"", ",", "\"\<11\>\"", ",", "\"\<00\>\""}], 
    "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This corresponds to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["1101", "2"], "=", "13"}], TraditionalForm]]],
 "."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["13.3 Finite-State Machines with No Output", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 "In this section we will see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to represent finite-state automata and to perform language recognition."
}], "Text"],

Cell[CellGroupData[{

Cell["Kleene Closure", "Subsection",
 CellTags->"Section3a"],

Cell["\<\
We begin this section by writing functions to compute the concatenation of \
two sets of strings and the partial Kleene closure of a set of strings. As in \
the previous section, we will model a string as a list.\
\>", "Text"],

Cell[TextData[{
 "Given two lists of strings (themselves represented as lists), we can form \
all possible concatenations by using ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " to concatenate each pair. In order to simplify the appearance of input, \
particularly to enter single-element strings as a simple number, this \
function will wrap any non-lists into a list structure so that single-term \
strings can be given to the function without braces."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setCat", "[", 
   RowBox[{"A_", ",", "B_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Flatten", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Which", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Head", "[", "a", "]"}], "===", "List"}], "&&", 
           RowBox[{
            RowBox[{"Head", "[", "b", "]"}], "===", "List"}]}], ",", 
          RowBox[{"Join", "[", 
           RowBox[{"a", ",", "b"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"Head", "[", "a", "]"}], "===", "List"}], "&&", 
           RowBox[{
            RowBox[{"Head", "[", "b", "]"}], "=!=", "List"}]}], ",", 
          RowBox[{"Join", "[", 
           RowBox[{"a", ",", 
            RowBox[{"{", "b", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"Head", "[", "a", "]"}], "=!=", "List"}], "&&", 
           RowBox[{
            RowBox[{"Head", "[", "b", "]"}], "===", "List"}]}], ",", 
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"{", "a", "}"}], ",", "b"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"Head", "[", "a", "]"}], "=!=", "List"}], "&&", 
           RowBox[{
            RowBox[{"Head", "[", "b", "]"}], "=!=", "List"}]}], ",", 
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"{", "a", "}"}], ",", 
            RowBox[{"{", "b", "}"}]}], "]"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"a", ",", "A"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"b", ",", "B"}], "}"}]}], "]"}], ",", "1"}], "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " is used since ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " with more than one loop specification produces a nested list. The argument \
1 prevents ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " from flattening the list beyond the highest level of nesting."
}], "Text"],

Cell["\<\
Applying this function to the sets from Example 1 produces the same output as \
in the solution to that example.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listA", "=", 
  RowBox[{"{", 
   RowBox[{"0", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listB", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "0"}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"setCat", "[", 
  RowBox[{"listA", ",", "listB"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Given a set ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", recall that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", "0"], TraditionalForm]]],
 " is defined to be the set of the empty string, and that for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", ">", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["A", 
     RowBox[{"n", "+", "1"}]], "=", 
    RowBox[{
     SuperscriptBox["A", "n"], "A"}]}], TraditionalForm]]],
 ". Also recall that the Kleene closure of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["A", "*"], "=", 
    RowBox[{
     StyleBox[
      UnderoverscriptBox["\[Union]", 
       RowBox[{" ", 
        RowBox[{"k", "=", "0"}]}], 
       RowBox[{" ", "\[Infinity]"}]],
      FontSize->16], 
     SuperscriptBox["A", "k"]}]}], TraditionalForm]]],
 ". We define the partial Kleene closure to level ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["A", 
     RowBox[{"[", "n", "]"}]], "=", 
    RowBox[{
     StyleBox[
      UnderoverscriptBox["\[Union]", 
       RowBox[{" ", 
        RowBox[{"k", "=", "0"}]}], 
       RowBox[{" ", "n"}]],
      FontSize->16], 
     SuperscriptBox["A", "k"]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We write the following function to produce the powers of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". The function is modeled on the recursive definition given in the text."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"setPow", "[", 
    RowBox[{"A_", ",", "k_"}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"k", "\[Equal]", "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "}"}], "}"}], ",", 
     RowBox[{"setCat", "[", 
      RowBox[{
       RowBox[{"setPow", "[", 
        RowBox[{"A", ",", 
         RowBox[{"k", "-", "1"}]}], "]"}], ",", "A"}], "]"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "For example, with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", "=", 
    RowBox[{"{", 
     RowBox[{"1", ",", "10", ",", "110"}], "}"}]}], TraditionalForm]]],
 ", we can compute ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["B", "3"], TraditionalForm]]],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"setPow", "[", 
  RowBox[{"listB", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "1", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "1", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1", ",", "1", ",", 
     "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "1", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "0", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "1", ",", 
     "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1", ",", 
     "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1", ",", "1",
      ",", "0"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To form the partial Kleene closure ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", 
    RowBox[{"[", "n", "]"}]], TraditionalForm]]],
 ", we must find the union of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["A", "0"], ",", 
    SuperscriptBox["A", "1"], ",", "\[Ellipsis]", ",", 
    SuperscriptBox["A", "n"]}], TraditionalForm]]],
 ". Iteratively building the ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", "k"], TraditionalForm]]],
 " while taking unions is more efficient than using ",
 StyleBox["setPow", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"kleene", "[", 
   RowBox[{"A_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"K", "=", 
       RowBox[{"{", 
        RowBox[{"{", "}"}], "}"}]}], ",", "x", ",", "Ak", ",", "i"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"K", "=", 
        RowBox[{"Union", "[", 
         RowBox[{"K", ",", 
          RowBox[{"{", 
           RowBox[{"{", "x", "}"}], "}"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "A"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Ak", "=", "K"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{"i", "\[LessEqual]", "n"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Ak", "=", 
         RowBox[{"setCat", "[", 
          RowBox[{"Ak", ",", "A"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"K", "=", 
         RowBox[{"Union", "[", 
          RowBox[{"K", ",", "Ak"}], "]"}]}]}]}], "\[IndentingNewLine]", "]"}],
      ";", "\[IndentingNewLine]", "K"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We compute the Kleene closure up to level 3 of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kleene", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Extended Transition Function for a Finite-State Automaton", "Subsection",
 CellTags->"Section3b"],

Cell["\<\
Now we will create a function that serves as the extension of the transition \
function of a finite-state automaton, as described following Example 4 in \
Section 13.3 of the text.\
\>", "Text"],

Cell["\<\
As in Section 13.2, we will model the transition function as an indexed \
variable. The indices will be the pairs consisting of the current state of \
the automaton and the input. The corresponding value will be the next state \
of the automaton.\
\>", "Text"],

Cell[TextData[{
 "For example, the transition function of the finite-state automaton ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["M", "1"], TraditionalForm]]],
 " in Example 5 is as follows."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ex51Table", "[", 
    RowBox[{"0", ",", "0"}], "]"}], "=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex51Table", "[", 
    RowBox[{"0", ",", "1"}], "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex51Table", "[", 
    RowBox[{"1", ",", "0"}], "]"}], "=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex51Table", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "=", "1"}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "To model the extended function that takes a pair consisting of a state and \
a member of the Kleene closure of the alphabet and returns the final state, \
we write a function, ",
 StyleBox["extendedTransition", "Input"],
 ". The arguments of this function will be a state number, a list \
representing the input string, and the transition function."
}], "Text"],

Cell["\<\
We will not use the recursive definition provided in the text, but will \
instead use an iterative approach. Begin by initializing the current state to \
the input state. Then loop through the list representing the input string and \
apply the transition function to update the current state. Once the loop is \
concluded, return the state.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"extendedTransition", "[", 
   RowBox[{"state_", ",", "input_", ",", "transFunc_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"curState", "=", "state"}], ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "input", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"curState", "=", 
        RowBox[{"transFunc", "[", 
         RowBox[{"curState", ",", 
          RowBox[{"input", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]",
       "]"}], ";", "\[IndentingNewLine]", "curState"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can use this function to see that applying the automaton ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["M", "1"], TraditionalForm]]],
 " from Example 5 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
   TraditionalForm]]],
 " from initial state ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 " ends in state ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"extendedTransition", "[", 
  RowBox[{"0", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
   "ex51Table"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Language Recognition with Finite-State Automata", "Subsection",
 CellTags->"Section3c"],

Cell[TextData[{
 "Recall that a string ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is recognized by a finite-state automaton if the extended transition \
function applied to the initial state and the string ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " results in a final state."
}], "Text"],

Cell[TextData[{
 "We will write a function that, given the transition table for a \
finite-state automaton with initial state ",
 StyleBox["init", "Input"],
 ", the set of ",
 StyleBox["final", "Input"],
 " states, and the string ",
 StyleBox["x", "Input"],
 ", will return ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " indicating whether or not the string is recognized by the machine."
}], "Text"],

Cell[TextData[{
 "The function only needs to apply ",
 StyleBox["extendedTransition", "Input"],
 " to the state ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 ", the transition table, and string, and then check to see whether or not \
the result is in the set of final states."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"recognizedQ", "[", 
   RowBox[{"x_", ",", "transFunc_", ",", "init_", ",", "final_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "endState", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"endState", "=", 
      RowBox[{"extendedTransition", "[", 
       RowBox[{"init", ",", "x", ",", "transFunc"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"MemberQ", "[", 
      RowBox[{"final", ",", "endState"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The solution to Example 5 indicated that the only strings accepted by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["M", "1"], TraditionalForm]]],
 " are those consisting of consecutive ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 "s."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"recognizedQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   "ex51Table", ",", "0", ",", 
   RowBox[{"{", "0", "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"recognizedQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", "ex51Table", ",",
    "0", ",", 
   RowBox[{"{", "0", "}"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using the ",
 StyleBox["kleene", "Input"],
 " function from the beginning of this section, we can partially determine \
the language recognized by a machine."
}], "Text"],

Cell[TextData[{
 "Given the transition table, the initial state, the set of final states, a \
set ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", and a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", the following function will calculate the subset of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", 
    RowBox[{"[", "n", "]"}]], TraditionalForm]]],
 " recognized by the finite-state automaton defined by the transition table \
and set of final states."
}], "Text"],

Cell[TextData[{
 "This function operates by brute force; applying ",
 StyleBox["kleene", "Input"],
 " and then using ",
 StyleBox["recognizedQ", "Input"],
 " to check each element of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", 
    RowBox[{"[", "n", "]"}]], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findLanguage", "[", 
   RowBox[{"transFunc_", ",", "init_", ",", "final_", ",", "A_", ",", "n_"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"An", ",", "x", ",", 
      RowBox[{"L", "=", 
       RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"An", "=", 
      RowBox[{"kleene", "[", 
       RowBox[{"A", ",", "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"recognizedQ", "[", 
          RowBox[{"x", ",", "transFunc", ",", "init", ",", "final"}], "]"}], 
         ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"L", ",", "x"}], "]"}]}], "]"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "An"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     "L"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying this function to our ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["M", "1"], TraditionalForm]]],
 " machine and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], 
    RowBox[{"[", "10", "]"}]], TraditionalForm]]],
 ", we see that the only strings in that set recognized by the finite-state \
automaton are those consisting only of 1s."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLanguage", "[", 
  RowBox[{"ex51Table", ",", "0", ",", 
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
     "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Nondeterministic Finite-State Automata", "Subsection",
 CellTags->"Section3d"],

Cell["\<\
We conclude this section with an implementation of the constructive proof of \
Theorem 1 of Section 13.3. Given a nondeterministic finite-state automaton, \
our function will produce a deterministic finite-state automaton.\
\>", "Text"],

Cell["\<\
In particular, given the transition function (indexed variable) for a \
nondeterministic automaton, its input alphabet, its starting state, and its \
set of final states, the function will produce the transition function for a \
deterministic automaton, its starting state, and its set of final states.\
\>", "Text"],

Cell["\<\
For a nondeterministic automaton, we will represent the transition function \
in the same way as for the deterministic automaton earlier, except the values \
for the variable will be sets of states, rather than individual states.\
\>", "Text"],

Cell["\<\
For example, here is the transition function for the nondeterministic \
automaton described in Example 10, which has final states 0 and 4.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"0", ",", "0"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"0", ",", "2"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"0", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "1", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"1", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "3", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "4", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"2", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "4", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"3", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "3", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"3", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"4", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "3", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ex10Table", "[", 
    RowBox[{"4", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "3", "}"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell["\<\
To determine the deterministic automaton's transition table, its starting \
state, and final states, we follow the proof of Theorem 1. The deterministic \
automaton's states are sets of states of the nondeterministic automaton. \
\>", "Text"],

Cell["\<\
We begin with the set consisting of the nondeterministic automaton's starting \
state. This is the starting state for the deterministic automaton. Given any \
state of the deterministic automaton, and any input, the deterministic \
transition is the union over all members of the state of the results of \
applying the nondeterministic automaton's transition with that input value.\
\>", "Text"],

Cell[TextData[{
 "In our function, we will create an indexed variable. We will also create \
two sets ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 ". The set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " will be initialized to the empty set and, at the conclusion of the \
procedure will be the set of all states of the deterministic automaton. The \
set ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " will be initialized to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"{", 
     SubscriptBox["s", "0"], "}"}], "}"}], TraditionalForm]]],
 ", the set containing the initial state of the deterministic automaton."
}], "Text"],

Cell[TextData[{
 "As long as ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " is non-empty, we will move one of its members from ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " and apply the nondeterministic automaton's transition function with all \
possible input values. The results are the entries in the deterministic \
transition table and those that are not already members of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " are added to ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " for further processing."
}], "Text"],

Cell["\<\
The final states of the deterministic automaton are those states which \
contain a final state of the nondeterministic automaton. That is, the final \
states are those whose intersection with the set of the original final states \
is nonempty. Before exiting, the function calculates the set of final states \
for the deterministic automaton.\
\>", "Text"],

Cell["\<\
Here is the function. Note that the function returns a list containing the \
new starting state, and the set of final states. Provided that we give the \
function a symbol that has not been assigned a value, the function will be \
assign values to it as an indexed variable storing the deterministic \
transition function.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeDeterministic", "[", 
   RowBox[{
   "newTable_Symbol", ",", "transFunc_", ",", "alphabet_", ",", "init_", ",", 
    "final_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"S", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"T", "=", 
       RowBox[{"{", 
        RowBox[{"{", "init", "}"}], "}"}]}], ",", "state", ",", "i", ",", "s",
       ",", "x", ",", "newfinal"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"T", "\[NotEqual]", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"state", "=", 
         RowBox[{"T", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"T", "=", 
         RowBox[{"Delete", "[", 
          RowBox[{"T", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"S", ",", "state"}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"x", "=", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"Do", "[", 
            RowBox[{
             RowBox[{"x", "=", 
              RowBox[{"Union", "[", 
               RowBox[{"x", ",", 
                RowBox[{"transFunc", "[", 
                 RowBox[{"s", ",", "i"}], "]"}]}], "]"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"s", ",", "state"}], "}"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"x", "=", 
            RowBox[{"Union", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"newTable", "[", 
             RowBox[{"state", ",", "i"}], "]"}], "=", "x"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"!", 
              RowBox[{"MemberQ", "[", 
               RowBox[{"S", ",", "x"}], "]"}]}], ",", 
             RowBox[{"T", "=", 
              RowBox[{"Union", "[", 
               RowBox[{"T", ",", 
                RowBox[{"{", "x", "}"}]}], "]"}]}]}], "]"}]}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "alphabet"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"newfinal", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Intersection", "[", 
           RowBox[{"state", ",", "final"}], "]"}], "\[NotEqual]", 
          RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"newfinal", "=", 
          RowBox[{"Union", "[", 
           RowBox[{"newfinal", ",", 
            RowBox[{"{", "state", "}"}]}], "]"}]}]}], "]"}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"state", ",", "S"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "init", "}"}], ",", "newfinal"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Applying this function to the Example 10 information produces the following.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ex10Dinit", ",", "ex10Dfinal"}], "}"}], "=", 
  RowBox[{"makeDeterministic", "[", 
   RowBox[{"ex10DTable", ",", "ex10Table", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "4"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "0", "}"}], ",", 
     RowBox[{"{", "4", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "4"}], "}"}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can inspect the transition table by applying ",
 StyleBox[ButtonBox["Definition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Definition"], "FunctionLink"],
 " or the ",
 StyleBox["?", "Input"],
 " operator to the symbol ",
 StyleBox["ex10DTable", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Definition", "[", "ex10DTable", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "0", "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "2"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "0", "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", "1", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "1", "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "1", "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", "4", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "3", "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "3", "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "4", "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", "4", "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "2"}], "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "2"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "2"}], "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", "4"}], "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", "4"}], "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "4"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"3", ",", "4"}], "}"}], ",", "0"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex10DTable", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"3", ",", "4"}], "}"}], ",", "1"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`ex10DTable],
  Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
You can confirm that this agrees with Figure 8 from Section 13.3. \
\>", "Text"],

Cell[TextData[{
 "We use the output as the arguments to ",
 StyleBox["findLanguage", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLanguage", "[", 
  RowBox[{"ex10DTable", ",", "ex10Dinit", ",", "ex10Dfinal", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "1", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "1", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
     "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
     "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "1", ",", 
     "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0",
      ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0",
      ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "1",
      ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0",
      ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0",
      ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0",
      ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
This list of strings suggests that the language recognized by this automaton \
are those strings consisting of a number of 0s followed by no more than two \
1's.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["13.4 Language Recognition", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "In this section we will introduce ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s support for regular expressions for working with strings. Note, however, \
that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s general patterns can also be used with strings. We will \
also develop a function for calculating the concatenation of two \
nondeterministic automata."
}], "Text"],

Cell[CellGroupData[{

Cell["Regular Expressions", "Subsection",
 CellTags->"Section4a"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", a regular expression is enclosed in the ",
 StyleBox[ButtonBox["RegularExpression",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RegularExpression"], "FunctionLink"],
 " head. It is typically used in the second argument to a string information \
or manipulation function, such as ",
 StyleBox[ButtonBox["StringMatchQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringMatchQ"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["StringReplace",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringReplace"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["StringCases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringCases"], "FunctionLink"],
 ", or ",
 StyleBox[ButtonBox["StringSplit",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringSplit"], "FunctionLink"],
 ". We will illustrate ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s syntax for regular expressions using the ",
 StyleBox[ButtonBox["StringMatchQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringMatchQ"], "FunctionLink"],
 " function. This function takes a string as the first argument and a regular \
expression, enclosed in ",
 StyleBox[ButtonBox["RegularExpression",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RegularExpression"], "FunctionLink"],
 ", as the second argument. It returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the regular expression matches the string."
}], "Text"],

Cell["\<\
Perhaps the most basic form of a regular expression is the concatenation of \
elements of the set. For example, \[OpenCurlyDoubleQuote]01\
\[CloseCurlyDoubleQuote] is a regular expression. This expression matches \
itself, of course.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<01\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<01\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
The output indicates that yes, the string \[OpenCurlyDoubleQuote]01\
\[CloseCurlyDoubleQuote] matches the regular expression \
\[OpenCurlyDoubleQuote]01\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell[CellGroupData[{

Cell["Kleene Closure", "Subsubsection"],

Cell["\<\
The asterisk is a symbol used in a regular expression to represent the Kleene \
closure.\
\>", "Text"],

Cell["\<\
For example, the regular expression \[OpenCurlyDoubleQuote]10*\
\[CloseCurlyDoubleQuote] will match a 1 followed by any number of 0s.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<10000000\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<10*\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<1\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<10*\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<0111000\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<10*\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
As in the text, parentheses can be used to group symbols. For example \
\[OpenCurlyDoubleQuote](10)*\[CloseCurlyDoubleQuote] matches any number of \
copies of \[OpenCurlyDoubleQuote]10\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<101010101010101010\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<(10)*\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<101010101\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<(10)*\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", and most languages that support regular expressions, also recognizes \
\[OpenCurlyDoubleQuote]+\[OpenCurlyDoubleQuote] and \[OpenCurlyDoubleQuote]?\
\[CloseCurlyDoubleQuote]. These are used like \[OpenCurlyDoubleQuote]*\
\[CloseCurlyDoubleQuote] but with different meaning. The expression \
\[OpenCurlyDoubleQuote]A+\[OpenCurlyDoubleQuote] is used to match one or more \
copies of \[OpenCurlyDoubleQuote]A\[CloseCurlyDoubleQuote]. Essentially, it \
is the Kleene closure minus the empty string. For example, \
\[OpenCurlyDoubleQuote]1*0+\[OpenCurlyDoubleQuote] matches any number of 1s \
followed by at least one 0."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<1111000\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1*0+\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<00\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1*0+\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<111\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1*0+\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
The \[OpenCurlyDoubleQuote]A?\[CloseCurlyDoubleQuote] expression is used to \
match 0 or 1 copies of \[OpenCurlyDoubleQuote]A\[CloseCurlyDoubleQuote]. For \
example, \[OpenCurlyDoubleQuote]1*0?\[CloseCurlyDoubleQuote] matches any \
number of 1s which may be followed by at most one 0.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<111111\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1*0?\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<1111110\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1*0?\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<11111100\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1*0?\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Union", "Subsubsection"],

Cell["\<\
To represent union, the vertical line is used. A \[OpenCurlyDoubleQuote]|\
\[CloseCurlyDoubleQuote] placed between two expressions will match either of \
them. The \[OpenCurlyDoubleQuote]|\[CloseCurlyDoubleQuote] can take the place \
of the \[OpenCurlyDoubleQuote]\[Union]\[CloseCurlyDoubleQuote] symbol in an \
expression such as \[OpenCurlyDoubleQuote]0(0\[Union]1)*\
\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<011010\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<0(0|1)*\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<1011010\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<0(0|1)*\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
This can also be done in more complicated expressions. For example, \
\[OpenCurlyDoubleQuote]2((10)*\[Union](01)*)2\[CloseCurlyDoubleQuote] \
describes the set of strings beginning and ending with 2s with an alternating \
sequence of 0s and 1s in between.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<21010102\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<2((10)*|(01)*)2\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<201012\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<2((10)*|(01)*)2\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<210012\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<2((10)*|(01)*)2\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
In some circumstances, union can be replaced by character classes. By placing \
characters within a pair of brackets, you indicate that any of the characters \
inside the brackets are allowed. For example, \[OpenCurlyDoubleQuote]0(0\
\[Union]1)*\[CloseCurlyDoubleQuote] can be expressed as follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<011010\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<0[01]*\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
Note that this is only allowed when the options are single characters.\
\>", "Text"],

Cell["\<\
Character classes can also be used to specify a range of characters with a \
hyphen. For example, \[OpenCurlyDoubleQuote](0\[Union]1\[Union]2\[Union]3\
\[Union]4)*\[CloseCurlyDoubleQuote] can be specified as follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<4213442101\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<[0-4]*\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
Character classes can be complemented. By beginning a character class with a \
caret, you indicate that any character other than those specified are \
allowed. For example, in the following, the regular expression matches all \
strings beginning with 1, ending with 0, and which include no other 1s nor 0s.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<169jwq0\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1[^01]*0\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<169j1wq0\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1[^01]*0\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
There are also several defined character classes: you enter \
\[OpenCurlyDoubleQuote]\\\\d\[CloseCurlyDoubleQuote] for a digit, \
\[OpenCurlyDoubleQuote]\\\\D\[CloseCurlyDoubleQuote] for a non-digit, \
\[OpenCurlyDoubleQuote]\\\\s\[CloseCurlyDoubleQuote] for space, including \
newline and tab, \[OpenCurlyDoubleQuote]\\\\S\[CloseCurlyDoubleQuote] for any \
non-whitespace character, \
\[OpenCurlyDoubleQuote]\\\\w\[CloseCurlyDoubleQuote] for a word character \
(letters, digits, and underscores), and \[OpenCurlyDoubleQuote]\\\\W\
\[CloseCurlyDoubleQuote] for a non-word character.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<126qb\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<\\\\d*\\\\w\\\\w\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<b32xy\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<\\\\d*\\\\w\\\\w\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
The special character dot, \[OpenCurlyDoubleQuote].\[CloseCurlyDoubleQuote], \
is used to match any character. For example, \[OpenCurlyDoubleQuote]1...0\
\[CloseCurlyDoubleQuote] will match any string beginning with a 1, followed \
by any three characters and ending with a 0.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<12340\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1...0\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<1230\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1...0\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringMatchQ", "[", 
  RowBox[{"\"\<1234567890\>\"", ",", 
   RowBox[{"RegularExpression", "[", "\"\<1...0\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Regular expressions in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " are extremely flexible. The interested reader is referred to the tutorial \
page on ",
 ButtonBox["regular expressions",
  BaseStyle->"Link",
  ButtonData->"paclet:tutorial/RegularExpressions"],
 " for more information."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Concatenation of Automata", "Subsection",
 CellTags->"Section4b"],

Cell["\<\
We will write a function that concatenates two nondeterministic finite-state \
automata, as described in the proof of Theorem 1 of the text.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Two Automata", "Subsubsection"],

Cell["\<\
We begin by defining two automata that our function will concatenate.\
\>", "Text"],

Cell["\<\
The first automata is the result of Example 3, for recognizing \
\[OpenCurlyDoubleQuote]1*\[Union]01\[CloseCurlyDoubleQuote]. Our \
implementation is based on the simple form shown in Figure3b.\
\>", "Text"],

Cell[TextData[{
 "Note that the diagram in the text omits the results of transitioning from \
certain states via certain input values. For example, it does not show the \
result of the transition from state ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["s", "1"], TraditionalForm]]],
 " with input 0. This makes for a simpler and cleaner diagram, but the \
transition table will need to include this information. It will be assumed \
that all such omissions correspond to a transition to the state ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", " ", "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
Here is the transition table corresponding to the automaton shown in Figure \
3b. \
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"0", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "2", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"0", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "1", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"1", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "1", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"2", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "3", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"3", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"atable", "[", 
    RowBox[{"3", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The final states for this automaton are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "3"}], "}"}], TraditionalForm]]],
 ". We can confirm that it recognizes \[OpenCurlyDoubleQuote]1*\[Union]01\
\[CloseCurlyDoubleQuote] by applying ",
 StyleBox["makeDeterministic", "Input"],
 " and ",
 StyleBox["findLanguage", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"aDinit", ",", "aDfinal"}], "}"}], "=", 
  RowBox[{"makeDeterministic", "[", 
   RowBox[{"aDtable", ",", "atable", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "0", "}"}], ",", 
     RowBox[{"{", "1", "}"}], ",", 
     RowBox[{"{", "3", "}"}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLanguage", "[", 
  RowBox[{"aDtable", ",", "aDinit", ",", "aDfinal", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
     "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
As you can see, the language recognized by this machine includes the string \
\[OpenCurlyDoubleQuote]01\[CloseCurlyDoubleQuote] as well as \
\[OpenCurlyDoubleQuote]1*\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell["\<\
The second automaton we create will recognize the language \
\[OpenCurlyDoubleQuote]101\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"0", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"0", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "1", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"1", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "2", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"2", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "3", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"3", ",", "0"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"btable", "[", 
    RowBox[{"3", ",", "1"}], "]"}], "=", 
   RowBox[{"{", "}"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell["\<\
The only final state is 3. We confirm that this models that machine that \
recognizes 101.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"bDinit", ",", "bDfinal"}], "}"}], "=", 
  RowBox[{"makeDeterministic", "[", 
   RowBox[{"bDtable", ",", "btable", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "0", ",", 
    RowBox[{"{", "3", "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "0", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"{", "3", "}"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLanguage", "[", 
  RowBox[{"bDtable", ",", "bDinit", ",", "bDfinal", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0", ",", "1"}], "}"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Concatenating the Machines", "Subsubsection"],

Cell["\<\
Our concatenation function will require the following arguments, for both \
machines: the transition table, the starting state, and the final states. It \
will also require that the two machines have a common input alphabet but that \
alphabet does not need to be an argument.\
\>", "Text"],

Cell["\<\
Recall the following elements of the construction of the concatenation as \
described in the proof of Theorem 1 of Section 13.4.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
The states of the concatenation is the union of the states of the original \
machines, which are assumed to be disjoint.\
\>", "Item1Numbered"],

Cell["\<\
The starting state of the concatenation is the starting state of the first of \
the two machines.\
\>", "Item1Numbered"],

Cell["\<\
The final states of the concatenation include the set of final states of the \
second machine.\
\>", "Item1Numbered"],

Cell["\<\
The final states of the concatenation also include the starting state if the \
empty string is a member of both languages.\
\>", "Item1Numbered"],

Cell["\<\
All transitions of the original machines are transitions of the new machine.\
\>", "Item1Numbered"],

Cell["\<\
Additionally, for every transition in the first machine leading to a final \
state, we add a transition in the concatenation to the starting state of the \
second machine.\
\>", "Item1Numbered"],

Cell["\<\
Finally, if the starting state of the first machine is final, then for every \
transition from the starting state of the second machine, we add a transition \
from the starting state of the new machine.\
\>", "Item1Numbered"]
}, Open  ]],

Cell["\<\
The assumption that the states of the original two machines are disjoint \
means that we will need to make them so. There are a variety of ways in which \
we could do this. Since we assume that states are designated by nonnegative \
integers, we can make the states distinct by multiplying each state by 10 and \
adding 1 if it is in the first machine and 2 if it is in the second machine.\
\>", "Text"],

Cell[TextData[{
 "Therefore, the starting state of the concatenation is found by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"10", "\[CenterDot]", 
     SubscriptBox["s", "A"]}], "+", "1"}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["s", "A"], TraditionalForm]]],
 " is the starting state of the first machine. In our example, this will be \
equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"10", "\[CenterDot]", "0"}], "+", "1"}], "=", "1"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Next, we find the final states of the concatenation. Let ",
 StyleBox["finalA", "Input"],
 " and ",
 StyleBox["finalB", "Input"],
 " be the sets of final states for the original two machines. According to \
point 3 above, the final states of the concatenated machine include the final \
states of the second machine. We only need to update the names."
}], "Text"],

Cell["\<\
The final states of the machines we defined above are as follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"finalA", "=", 
  RowBox[{"{", 
   RowBox[{"0", ",", "1", ",", "3"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"finalB", "=", 
  RowBox[{"{", "3", "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", "3", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can obtain the final states of the concatenation by applying the \
function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "\[Rule]", 
    RowBox[{
     RowBox[{"10", "s"}], "+", "2"}]}], TraditionalForm]]],
 " to the set of final states of the second machine."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"10", "*", "#"}], "+", "2"}], ")"}], "&"}], ",", "finalB"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "32", "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Item 4 asserts that the starting state of the concatenated machine is a final \
state if and only if the empty string is a member of both languages. Another \
way to put this is that the starting state of the concatenated machine is a \
final state when both of the original machines have their own starting states \
as final states. This is not the case in our example. We will include this \
possibility in our general function by checking to see if the starting states \
are members of the sets of final states.\
\>", "Text"],

Cell[TextData[{
 "To form the transitions of the new machine, we begin with an unassigned \
variable. We will apply ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " to be certain nothing has been assigned."
}], "Text"],

Cell[BoxData[
 RowBox[{"Clear", "[", "abtable", "]"}]], "Input"],

Cell[TextData[{
 "We proceed as follows. First we need to obtain a list of all indices in the \
table ",
 StyleBox["atable", "Input"],
 ". To do this, we use ",
 StyleBox[ButtonBox["DownValues",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DownValues"], "FunctionLink"],
 " to obtain a list of the rules that make up the indexed variable. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DownValues", "[", "atable", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"0", ",", "0"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "2", "}"}]}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"0", ",", "1"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "1", "}"}]}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"1", ",", "0"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "1", "}"}]}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"2", ",", "0"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "3", "}"}]}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"3", ",", "0"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", 
      RowBox[{"3", ",", "1"}], "]"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"{", "}"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can extract the indices by applying ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") and ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"indicesA", "=", 
  RowBox[{
   RowBox[{"DownValues", "[", "atable", "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"All", ",", "1"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"0", ",", "0"}], "]"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"0", ",", "1"}], "]"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"1", ",", "0"}], "]"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"2", ",", "0"}], "]"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"3", ",", "0"}], "]"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"atable", "[", 
     RowBox[{"3", ",", "1"}], "]"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"indicesA", "=", 
  RowBox[{"indicesA", "/.", 
   RowBox[{
    RowBox[{"HoldPattern", "[", 
     RowBox[{"atable", "[", "x__", "]"}], "]"}], "\[Rule]", 
    RowBox[{"{", "x", "}"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", "0"}], "}"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", "1"}], "}"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"3", ",", "0"}], "}"}], "]"}], ",", 
   RowBox[{"HoldPattern", "[", 
    RowBox[{"{", 
     RowBox[{"3", ",", "1"}], "}"}], "]"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"indicesA", "=", 
  RowBox[{"indicesA", "[", 
   RowBox[{"[", 
    RowBox[{"All", ",", "1"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We create a general function following this pattern which we call ",
 StyleBox["getIndices", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"getIndices", "[", "indexedV_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"indices", ",", "k"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"indices", "=", 
      RowBox[{
       RowBox[{"DownValues", "[", "indexedV", "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"indices", "=", 
      RowBox[{"indices", "/.", 
       RowBox[{
        RowBox[{"HoldPattern", "[", 
         RowBox[{"indexedV", "[", "k__", "]"}], "]"}], "\[Rule]", 
        RowBox[{"{", "k", "}"}]}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"indices", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "1"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "For each index ",
 StyleBox["i", "Input"],
 ", the index in ",
 StyleBox["abtable", "Input"],
 " will be ",
 StyleBox["[10*i[[1]]+1,i[[2]]]", "Input"],
 ". This computes the appropriate state name in the concatenated machine and \
keeps the same input value. The associated entry will be obtained by using ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " and the pure function ",
 StyleBox["10*#+1", "Input"],
 " applied to the previous value. Note that we ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") ",
 StyleBox["atable", "Input"],
 " to ",
 StyleBox["i", "Input"],
 " to replace the ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 " head with ",
 StyleBox["atable", "Input"],
 " in order to access the value."
}], "Text"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"abtable", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"10", "*", 
        RowBox[{"i", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "+", "1"}], ",", 
      RowBox[{"i", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"10", "*", "#"}], "+", "1"}], ")"}], "&"}], ",", 
      RowBox[{"atable", "@@", "i"}]}], "]"}]}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "indicesA"}], "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can now inspect the values of ",
 StyleBox["abtable", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "abtable"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`abtable", "Print", "PrintUsage",
 CellTags->"Info3580720248-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "21", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "11", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "11", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "31", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`abtable],
  Editable->False]], "Print",
 CellTags->"Info3580720248-4013843"]
}, Open  ]]
}, Open  ]],

Cell["\<\
For the second machine, we do the same thing except adding 2 instead of 1.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"indicesB", "=", 
   RowBox[{"getIndices", "[", "btable", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"abtable", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"10", "*", 
        RowBox[{"i", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "+", "2"}], ",", 
      RowBox[{"i", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"10", "*", "#"}], "+", "2"}], ")"}], "&"}], ",", 
      RowBox[{"btable", "@@", "i"}]}], "]"}]}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "indicesB"}], "}"}]}], "]"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "abtable"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`abtable", "Print", "PrintUsage",
 CellTags->"Info3580720248-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "21", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "11", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"2", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "12", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "11", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"12", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "22", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"12", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "31", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"22", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"22", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "32", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"32", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"32", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`abtable],
  Editable->False]], "Print",
 CellTags->"Info3580720248-4013843"]
}, Open  ]]
}, Open  ]],

Cell["\<\
Next, we must add transitions between the two components. As item 6 \
instructs, for each transition in the first of the two machines that leads to \
a final state, we must add a transition in the concatenated machine to the \
starting state of the second machine.\
\>", "Text"],

Cell[TextData[{
 "We will again loop through the indices of ",
 StyleBox["atable", "Input"],
 ", this time checking whether the image contains any states that are final \
for machine A. If so, we will add the transition to state 2 (the name of the \
starting state in the second machine in the concatenation). (Note that we \
must update the entry in the ",
 StyleBox["abtable", "Input"],
 " rather than replace it.)"
}], "Text"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Intersection", "[", 
       RowBox[{
        RowBox[{"atable", "@@", "i"}], ",", "finalA"}], "]"}], "\[NotEqual]", 
      
      RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"abtable", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"10", "*", 
          RowBox[{"i", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], "+", "1"}], ",", 
        RowBox[{"i", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
      RowBox[{"Union", "[", 
       RowBox[{
        RowBox[{"abtable", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"10", "*", 
            RowBox[{"i", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "+", "1"}], ",", 
          RowBox[{"i", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
        RowBox[{"{", "2", "}"}]}], "]"}]}]}], "]"}], "\[IndentingNewLine]", 
   ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "indicesA"}], "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell["We can see that this has added transitions to state 2.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "abtable"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`abtable", "Print", "PrintUsage",
 CellTags->"Info3580720248-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "21", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "11"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"2", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "12", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "11"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"12", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "22", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"12", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "31"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"22", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"22", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "32", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"32", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"32", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`abtable],
  Editable->False]], "Print",
 CellTags->"Info3580720248-4013843"]
}, Open  ]]
}, Open  ]],

Cell["\<\
Finally, since the starting state of the first machine is final, we must add \
transitions from the starting state of the concatenated machine for each of \
the transitions from the starting state of the second machine. The starting \
state of the second machine in this example is 0, and the starting state of \
the concatenation is 1.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"i", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", "0"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"abtable", "[", 
       RowBox[{"1", ",", 
        RowBox[{"i", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", "\[IndentingNewLine]", 
      
      RowBox[{"Union", "[", 
       RowBox[{
        RowBox[{"abtable", "[", 
         RowBox[{"1", ",", 
          RowBox[{"i", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"10", "*", "#"}], "+", "2"}], ")"}], "&"}], ",", 
          RowBox[{"btable", "@@", "i"}]}], "]"}]}], "]"}]}]}], "]"}], 
   "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "indicesB"}], "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell["Inspect the table again.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "abtable"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`abtable", "Print", "PrintUsage",
 CellTags->"Info3580720249-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "21", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "11", ",", "12"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"2", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "12", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"11", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "11"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"12", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "22", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"12", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"21", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "31"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"22", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"22", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "32", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"31", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"32", ",", "0"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"abtable", "[", 
          RowBox[{"32", ",", "1"}], "]"}], "=", 
         RowBox[{"{", "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`abtable],
  Editable->False]], "Print",
 CellTags->"Info3580720249-4013843"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "Note that this modified the entry for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], TraditionalForm]]],
 ". (Recall that state 1 is the starting state for the combined machine.) \
Before, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], TraditionalForm]]],
 " was associated with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"2", ",", "11"}], "}"}], TraditionalForm]]],
 ", the starting state of the second machine and state 1 of the first \
machine. Now, the entry for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], TraditionalForm]]],
 " also includes 12, state 1 of the second machine."
}], "Text"],

Cell[TextData[{
 "That ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], TraditionalForm]]],
 " is associated with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"2", ",", "11", ",", "12"}], "}"}], TraditionalForm]]],
 " means that from the starting state of the concatenation and input 1, there \
are three options. Going to state 2, the starting state of the second \
machine, corresponds to recognizing the string 1 followed by a string \
recognized by the second machine. Going to state 11, state 1 of the first \
machine, corresponds to building a string of all 1s, which is recognized by \
the first machine. And going to state 12, state 1 of the second machine, \
corresponds to the first machine contributing the empty string followed by 1 \
as the first character of a string recognized by the second machine."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementation as a Function", "Subsubsection"],

Cell["Here is the complete function based on the example above.", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"catAutomata", ",", 
    RowBox[{"{", "HoldFirst", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"catAutomata", "[", 
   RowBox[{
   "abtable_", ",", "atable_", ",", "astart_", ",", "afinal_", ",", "btable_",
     ",", "bstart_", ",", "bfinal_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "abstart", ",", "abfinal", ",", "indicesA", ",", "indicesB", ",", "i"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"abstart", "=", 
      RowBox[{
       RowBox[{"10", "*", "astart"}], "+", "1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"abfinal", "=", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"10", "*", "#"}], "+", "2"}], ")"}], "&"}], ",", 
        "bfinal"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"MemberQ", "[", 
         RowBox[{"afinal", ",", "astart"}], "]"}], "&&", 
        RowBox[{"MemberQ", "[", 
         RowBox[{"bfinal", ",", "bstart"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"abfinal", "=", 
        RowBox[{"Union", "[", 
         RowBox[{"abfinal", ",", 
          RowBox[{"{", "abstart", "}"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Clear", "[", "abtable", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"indicesA", "=", 
      RowBox[{"getIndices", "[", "atable", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"indicesB", "=", 
      RowBox[{"getIndices", "[", "btable", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"abtable", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"10", "*", 
            RowBox[{"i", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "+", "1"}], ",", 
          RowBox[{"i", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"10", "*", "#"}], "+", "1"}], ")"}], "&"}], ",", 
          RowBox[{"atable", "@@", "i"}]}], "]"}]}], "\[IndentingNewLine]", 
       ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "indicesA"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"abtable", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"10", "*", 
            RowBox[{"i", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "+", "2"}], ",", 
          RowBox[{"i", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"10", "*", "#"}], "+", "2"}], ")"}], "&"}], ",", 
          RowBox[{"btable", "@@", "i"}]}], "]"}]}], "\[IndentingNewLine]", 
       ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "indicesB"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Intersection", "[", 
           RowBox[{
            RowBox[{"atable", "@@", "i"}], ",", "finalA"}], "]"}], 
          "\[NotEqual]", 
          RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"abtable", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"10", "*", 
              RowBox[{"i", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "+", "1"}], ",", 
            RowBox[{"i", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
          RowBox[{"Union", "[", 
           RowBox[{
            RowBox[{"abtable", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"10", "*", 
                RowBox[{"i", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], "+", "1"}], ",", 
              RowBox[{"i", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
            RowBox[{"{", "2", "}"}]}], "]"}]}]}], "]"}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "indicesA"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"MemberQ", "[", 
        RowBox[{"afinal", ",", "astart"}], "]"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"i", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", "0"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"abtable", "[", 
             RowBox[{"1", ",", 
              RowBox[{"i", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
            "\[IndentingNewLine]", 
            RowBox[{"Union", "[", 
             RowBox[{
              RowBox[{"abtable", "[", 
               RowBox[{"1", ",", 
                RowBox[{"i", "[", 
                 RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
              RowBox[{"Map", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"10", "*", "#"}], "+", "2"}], ")"}], "&"}], ",", 
                RowBox[{"btable", "@@", "i"}]}], "]"}]}], "]"}]}]}], "]"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "indicesB"}], "}"}]}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"abstart", ",", "abfinal"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying this to our examples and passing the results on to ",
 StyleBox["makeDeterministic", "Input"],
 " and ",
 StyleBox["findLanguage", "Input"],
 " shows us that the result does indeed recognize \[OpenCurlyDoubleQuote](1*\
\[Union]01)101\[CloseCurlyDoubleQuote]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cstart", ",", "cfinal"}], "}"}], "=", 
  RowBox[{"catAutomata", "[", 
   RowBox[{"ctable", ",", "atable", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", "3"}], "}"}], ",", "btable", ",", "0", ",", 
    RowBox[{"{", "3", "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", 
   RowBox[{"{", "32", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cDstart", ",", "cDfinal"}], "}"}], "=", 
  RowBox[{"makeDeterministic", "[", 
   RowBox[{"cDtable", ",", "ctable", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "cstart", ",", "cfinal"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"{", "32", "}"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLanguage", "[", 
  RowBox[{"cDtable", ",", "cDstart", ",", "cDfinal", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", 
     "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0",
      ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "0", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["13.5 Turing Machines", "Section",
 CellTags->"Section5"],

Cell[TextData[{
 "In this section we will explore ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["TuringMachine",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TuringMachine"], "FunctionLink"],
 " function. We will then create our own model of a Turing machine to help \
you better understand this important concept in detail."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["TuringMachine", "Input"]], "Subsection",
 CellTags->"Section5a"],

Cell[TextData[{
 "To illustrate ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in function, we will use Example 1 from Section \
13.5. This Turing machine is defined by seven tuples: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["s", "0"], ",", "0", ",", 
     SubscriptBox["s", "0"], ",", "0", ",", "R"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["s", "0"], ",", "1", ",", 
     SubscriptBox["s", "1"], ",", "1", ",", "R"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["s", "0"], ",", "B", ",", 
     SubscriptBox["s", "3"], ",", "B", ",", "R"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["s", "1"], ",", "0", ",", 
     SubscriptBox["s", "0"], ",", "0", ",", "R"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["s", "1"], ",", "1", ",", 
     SubscriptBox["s", "2"], ",", "0", ",", "L"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["s", "1"], ",", "B", ",", 
     SubscriptBox["s", "3"], ",", "B", ",", "R"}], ")"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["s", "2"], ",", "1", ",", 
     SubscriptBox["s", "3"], ",", "0", ",", "R"}], ")"}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "The first argument of ",
 StyleBox[ButtonBox["TuringMachine",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TuringMachine"], "FunctionLink"],
 " will be that data, but in the form of ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 "s (",
 StyleBox["->", "Input"],
 ") of the form ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     StyleBox["state", "CodeVar"], ",", 
     StyleBox["entry", "CodeVar"]}], "}"}], "->", 
   RowBox[{"{", 
    RowBox[{
     StyleBox["newstate", "CodeVar"], ",", 
     StyleBox["newentry", "CodeVar"], ",", 
     StyleBox["move", "CodeVar"]}], "}"}]}]], "myCode"],
 ", where ",
 StyleBox["state", "CodeVar"],
 " and ",
 StyleBox["entry", "CodeVar"],
 " are the current state of the machine and the value seen by the head and ",
 StyleBox["newstate", "CodeVar"],
 " and ",
 StyleBox["newentry", "CodeVar"],
 " are the next state and the value to be written on the tape. The ",
 StyleBox["move", "CodeVar"],
 " is an integer representing how the head is to move, with +1 representing \
right and -1 left. So the machine of Example 1 is described by the \
following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"example1Rules", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "\"\<\>\""}], "}"}], "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"3", ",", "\"\<\>\"", ",", "1"}], "}"}]}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}], "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"2", ",", "0", ",", 
       RowBox[{"-", "1"}]}], "}"}]}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "\"\<\>\""}], "}"}], "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"3", ",", "\"\<\>\"", ",", "1"}], "}"}]}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "1"}], "}"}], "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"3", ",", "0", ",", "1"}], "}"}]}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "\<\"\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"3", ",", "\<\"\"\>", ",", "1"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"2", ",", "0", ",", 
      RowBox[{"-", "1"}]}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "\<\"\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"3", ",", "\<\"\"\>", ",", "1"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "1"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"3", ",", "0", ",", "1"}], "}"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second argument to ",
 StyleBox[ButtonBox["TuringMachine",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TuringMachine"], "FunctionLink"],
 " specifies the initial conditions. It is a list containing two members. The \
first element of the initial conditions list will be the initial state of the \
machine. The second element of the initial conditions is a list with two \
members, the first being a list representing a finite portion of the tape and \
the second specifying the value appearing at every position of the infinite \
tape outside the finite area. "
}], "Text"],

Cell[TextData[{
 "In our example, the machine will begin in state 0. The tape is initally \
{0,1,0,1,1,0} with blanks outside that range. So the second argument to ",
 StyleBox[ButtonBox["TuringMachine",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TuringMachine"], "FunctionLink"],
 " will be "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"example1Init", "=", 
  RowBox[{"{", 
   RowBox[{"0", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}],
       ",", "\"\<\>\""}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
     ",", "\<\"\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["TuringMachine",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TuringMachine"], "FunctionLink"],
 " to these two elements produces the following output:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TuringMachine", "[", 
  RowBox[{"example1Rules", ",", "example1Init"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
     ",", "\<\"\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
This output represents the result of one step of the Turing machine. It is of \
the form\
\>", "Text"],

Cell[TextData[{
 " ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"{", "{"}], 
   StyleBox["state", "CodeVar"], ",", 
   StyleBox["pos", "CodeVar"], ",", 
   StyleBox["distance", "CodeVar"], 
   RowBox[{"}", ",", "{"}], 
   StyleBox["tape", "CodeVar"], ",", 
   StyleBox["rest", "CodeVar"], 
   RowBox[{"}", "}"}]}]], "myCode"]
}], "Text"],

Cell[TextData[{
 "where ",
 StyleBox["state", "CodeVar"],
 " is the new state of the machine, ",
 StyleBox["tape", "CodeVar"],
 " is the current state of the finite segment of tape with ",
 StyleBox["rest", "CodeVar"],
 " filling the rest of the infinite tape, ",
 StyleBox["pos", "CodeVar"],
 " is the position of the head relative to the list ",
 StyleBox["tape", "CodeVar"],
 ", and ",
 StyleBox["distance", "CodeVar"],
 " is how far the head has moved from its starting position. So the output \
above indicates that the machine is still in state 0 but has moved one \
position to the left."
}], "Text"],

Cell["\<\
Note that you can initialize a machine with a position argument similar to \
this output, but without the distance. The following will start the machine \
at the final 1 of the tape.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TuringMachine", "[", 
  RowBox[{"example1Rules", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
        "}"}], ",", "\"\<\>\""}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "6", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
     ",", "\<\"\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that the machine has moved one position to the left and changed to state \
1.\
\>", "Text"],

Cell["\<\
An optional third argument allows you run the machine more than one step.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TuringMachine", "[", 
  RowBox[{"example1Rules", ",", "example1Init", ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
      "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "2", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
      "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "3", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
      "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "4", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
      "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "5", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
      "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "4", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "0", ",", "0"}], 
      "}"}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that the output is a list of lists representing each step along the way. \
The final element indicates that after 5 steps, the machine is in state 2 at \
position 4.\
\>", "Text"],

Cell[TextData[{
 "For a machine with a terminal state, we can run it to completion with a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop as below. Note that ",
 StyleBox[ButtonBox["TuringMachine",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TuringMachine"], "FunctionLink"],
 " allows its initialization argument to include the distance parameter, so \
that we can feed its output back to it."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"machinestate", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
        "}"}], ",", "\"\<\>\""}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"While", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"machinestate", "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "1"}], "]"}], "]"}], "\[NotEqual]", "3"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"machinestate", "=", 
     RowBox[{"TuringMachine", "[", 
      RowBox[{"example1Rules", ",", "machinestate"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}], 
  ";"}], "\[IndentingNewLine]", "machinestate"}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], 
     ",", "\<\"\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that this agrees with the result of Example 1 in the textbook.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Creating a Turing Machine Function", "Subsection",
 CellTags->"Section5b"],

Cell[TextData[{
 "In our model, the tape will be represented by a list, with the assumption \
that all elements to the left and right of the bounds of the list are blanks. \
The blank symbol will be represented by the symbol ",
 StyleBox["B", "Input"],
 " and left and right by the symbols ",
 StyleBox["L", "Input"],
 " and ",
 StyleBox["R", "Input"],
 ". We ensure that these have not been assigned values by applying ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"B", ",", "L", ",", "R"}], "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["The Partial Function", "Subsubsection"],

Cell[TextData[{
 "The text uses the convention that the partial function that controls the \
operation of the Turing machine is defined by a set of five-tuples. It will \
be more convenient for our functions to represent the partial function as an \
indexed variable whose indices are pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"s", ",", "x"}], "]"}], TraditionalForm]]],
 " and whose values are triples ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s", "'"}], ",", 
     RowBox[{"x", "'"}], ",", "d"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
We create a function that will transform the set of 5-tuples representation \
into the indexed variable representation. The indexed variable to be defined \
is given as the first argument.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tuplesToIndexed", "[", 
   RowBox[{"indexedV_Symbol", ",", "S_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "x", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Clear", "[", "indexedV", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"indexedV", "[", 
         RowBox[{
          RowBox[{"x", "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", 
          RowBox[{"x", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
        RowBox[{"x", "[", 
         RowBox[{"[", 
          RowBox[{"{", 
           RowBox[{"3", ",", "4", ",", "5"}], "}"}], "]"}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "S"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Applying this function to the set of tuples given in Example 1 provides us \
with an example of a partial function to work with.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"tuplesToIndexed", "[", 
  RowBox[{"ex1", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1", ",", "1", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "B", ",", "3", ",", "B", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "2", ",", "0", ",", "L"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "B", ",", "3", ",", "B", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "1", ",", "3", ",", "0", ",", "R"}], "}"}]}], "}"}]}],
   "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "ex1"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`ex1", "Print", "PrintUsage",
 CellTags->"Info3580720250-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"ex1", "[", 
          RowBox[{"0", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex1", "[", 
          RowBox[{"0", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex1", "[", 
          RowBox[{"0", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "B", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex1", "[", 
          RowBox[{"1", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex1", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "0", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex1", "[", 
          RowBox[{"1", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "B", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex1", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "0", ",", "R"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`ex1],
  Editable->False]], "Print",
 CellTags->"Info3580720250-4013843"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox["B", "Input"],
 ", ",
 StyleBox["L", "Input"],
 ", and ",
 StyleBox["R", "Input"],
 " must all be unassigned symbols, otherwise they will be evaluated within \
the set of 5-tuples and will produce unexpected results."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Turing Machine Function", "Subsubsection"],

Cell["\<\
Our Turing machine function will accept as input an indexed variable \
representing the partial function, a list representing the status of the tape \
before running the machine, and the initial state. It will return the final \
tape and the final state.\
\>", "Text"],

Cell[TextData[{
 "When the function begins, we initialize the symbol ",
 StyleBox["pos", "Input"],
 " to 1, indicating that the control head is positioned at the leftmost \
element in the tape. We set the ",
 StyleBox["state", "Input"],
 " of the machine to the initial state and copy the ",
 StyleBox["tape", "Input"],
 " from the argument as well. We also compute the ",
 StyleBox["domain", "Input"],
 " of the partial function using the ",
 StyleBox["getIndices", "Input"],
 " function we created in the previous section. This will make it easier to \
check whether we have reached a halt."
}], "Text"],

Cell[TextData[{
 "The main work of the function will take place within a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop controlled by the condition that the domain of the function includes \
the pair consisting of the current state and the entry on the tape at the \
current position."
}], "Text"],

Cell[TextData[{
 "Within the loop, we first obtain the values of the new state, new tape \
entry, and direction from the partial function. We then set the ",
 StyleBox["state", "Input"],
 " to the new state, change the entry on the ",
 StyleBox["tape", "Input"],
 ", and update the position ",
 StyleBox["pos", "Input"],
 ". Note that when changing the position of the control head, we must take \
care not to exceed the bounds of the list representing the tape. If the \
previous position was location 1 in the list and the direction is left, then \
instead of changing the position, we extend the list by adding a blank on the \
left with the ",
 StyleBox[ButtonBox["PrependTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrependTo"], "FunctionLink"],
 " function. On the other hand, if the previous position was the right end of \
the tape and the direction is right, then we increase the position and extend \
the tape to the right via ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 "."
}], "Text"],

Cell["Here is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Turing", "[", 
   RowBox[{"f_", ",", "t_", ",", "init_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"pos", "=", "1"}], ",", 
      RowBox[{"state", "=", "init"}], ",", 
      RowBox[{"tape", "=", "t"}], ",", "domain", ",", "y"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"domain", "=", 
      RowBox[{"getIndices", "[", "f", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"MemberQ", "[", 
        RowBox[{"domain", ",", 
         RowBox[{"{", 
          RowBox[{"state", ",", 
           RowBox[{"tape", "[", 
            RowBox[{"[", "pos", "]"}], "]"}]}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "=", 
         RowBox[{"f", "[", 
          RowBox[{"state", ",", 
           RowBox[{"tape", "[", 
            RowBox[{"[", "pos", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"state", "=", 
         RowBox[{"y", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"tape", "[", 
          RowBox[{"[", "pos", "]"}], "]"}], "=", 
         RowBox[{"y", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Which", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"pos", "\[Equal]", "1"}], "&&", 
           RowBox[{
            RowBox[{"y", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "===", "L"}]}], ",", 
          RowBox[{"PrependTo", "[", 
           RowBox[{"tape", ",", "B"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"pos", "\[Equal]", 
            RowBox[{"Length", "[", "tape", "]"}]}], "&&", 
           RowBox[{
            RowBox[{"y", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "===", "R"}]}], ",", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"tape", ",", "B"}], "]"}], ";", 
           RowBox[{"pos", "++"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"y", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "===", "L"}], ",", 
          RowBox[{"pos", "--"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"y", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "===", "R"}], ",", 
          RowBox[{"pos", "++"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"tape", ",", "state"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We use the function to run the Turing machine from Example 1 on the tape \
shown in Figure 2a.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Turing", "[", 
  RowBox[{"ex1", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
   ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], 
   ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Observe that this agrees with Figure 2 from Section 13.5 in the text.\
\>", "Text"],

Cell[TextData[{
 "We create a verbose version of this function as well. The operation of the \
verbose version is identical to ",
 StyleBox["Turing", "Input"],
 ", but it displays the status of the machine at every step."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"verboseTuring", "[", 
   RowBox[{"f_", ",", "t_", ",", "init_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"pos", "=", "1"}], ",", 
      RowBox[{"state", "=", "init"}], ",", 
      RowBox[{"tape", "=", "t"}], ",", "domain", ",", "y", ",", 
      "displayTape"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"domain", "=", 
      RowBox[{"getIndices", "[", "f", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"displayTape", "=", "t"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"displayTape", "[", 
       RowBox[{"[", "pos", "]"}], "]"}], "=", 
      RowBox[{"\"\<\[Rule]\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"tape", "[", 
         RowBox[{"[", "pos", "]"}], "]"}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"displayTape", ",", "state"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"MemberQ", "[", 
        RowBox[{"domain", ",", 
         RowBox[{"{", 
          RowBox[{"state", ",", 
           RowBox[{"tape", "[", 
            RowBox[{"[", "pos", "]"}], "]"}]}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "=", 
         RowBox[{"f", "[", 
          RowBox[{"state", ",", 
           RowBox[{"tape", "[", 
            RowBox[{"[", "pos", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"state", "=", 
         RowBox[{"y", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"tape", "[", 
          RowBox[{"[", "pos", "]"}], "]"}], "=", 
         RowBox[{"y", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Which", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"pos", "\[Equal]", "1"}], "&&", 
           RowBox[{
            RowBox[{"y", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "===", "L"}]}], ",", 
          RowBox[{"PrependTo", "[", 
           RowBox[{"tape", ",", "B"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"pos", "\[Equal]", 
            RowBox[{"Length", "[", "tape", "]"}]}], "&&", 
           RowBox[{
            RowBox[{"y", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "===", "R"}]}], ",", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"tape", ",", "B"}], "]"}], ";", 
           RowBox[{"pos", "++"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"y", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "===", "L"}], ",", 
          RowBox[{"pos", "--"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"y", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "===", "R"}], ",", 
          RowBox[{"pos", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"displayTape", "=", "tape"}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"displayTape", "[", 
          RowBox[{"[", "pos", "]"}], "]"}], "=", 
         RowBox[{"\"\<\[Rule]\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"tape", "[", 
            RowBox[{"[", "pos", "]"}], "]"}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"displayTape", ",", "state"}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"tape", ",", "state"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"verboseTuring", "[", 
  RowBox[{"ex1", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
   ",", "0"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"\[Rule]0\"\>", ",", "1", ",", "0", ",", "1", ",", "1", ",", 
     "0"}], "}"}], "\[InvisibleSpace]", "0"}],
  SequenceForm[{"\[Rule]0", 1, 0, 1, 1, 0}, 0],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0", ",", "\<\"\[Rule]1\"\>", ",", "0", ",", "1", ",", "1", ",", "0"}], 
    "}"}], "\[InvisibleSpace]", "0"}],
  SequenceForm[{0, "\[Rule]1", 0, 1, 1, 0}, 0],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0", ",", "1", ",", "\<\"\[Rule]0\"\>", ",", "1", ",", "1", ",", "0"}], 
    "}"}], "\[InvisibleSpace]", "1"}],
  SequenceForm[{0, 1, "\[Rule]0", 1, 1, 0}, 1],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0", ",", "1", ",", "0", ",", "\<\"\[Rule]1\"\>", ",", "1", ",", "0"}], 
    "}"}], "\[InvisibleSpace]", "0"}],
  SequenceForm[{0, 1, 0, "\[Rule]1", 1, 0}, 0],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0", ",", "1", ",", "0", ",", "1", ",", "\<\"\[Rule]1\"\>", ",", "0"}], 
    "}"}], "\[InvisibleSpace]", "1"}],
  SequenceForm[{0, 1, 0, 1, "\[Rule]1", 0}, 1],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0", ",", "1", ",", "0", ",", "\<\"\[Rule]1\"\>", ",", "0", ",", "0"}], 
    "}"}], "\[InvisibleSpace]", "2"}],
  SequenceForm[{0, 1, 0, "\[Rule]1", 0, 0}, 2],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0", ",", "1", ",", "0", ",", "0", ",", "\<\"\[Rule]0\"\>", ",", "0"}], 
    "}"}], "\[InvisibleSpace]", "3"}],
  SequenceForm[{0, 1, 0, 0, "\[Rule]0", 0}, 3],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], 
   ",", "3"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Applications of Turing Machines", "Subsection",
 CellTags->"Section5c"],

Cell["\<\
We now apply our Turing machine function to two applications: recognizing \
strings in a language and computing functions.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Recognizing Sets", "Subsubsection"],

Cell[TextData[{
 "We will implement the Turing machine for recognizing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox["0", "n"], 
      SuperscriptBox["1", "n"]}], "|", 
     RowBox[{"n", "\[GreaterEqual]", "1"}]}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
The partial function was given in the solution to Example 3. To be safe, we \
again clear all the symbols used.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"M", ",", "B", ",", "L", ",", "R"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"tuplesToIndexed", "[", 
  RowBox[{"ex3", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1", ",", "M", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "M", ",", "2", ",", "M", ",", "L"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "B", ",", "2", ",", "B", ",", "L"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "1", ",", "3", ",", "M", ",", "L"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "1", ",", "3", ",", "1", ",", "L"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "0", ",", "4", ",", "0", ",", "L"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "M", ",", "5", ",", "M", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "0", ",", "4", ",", "0", ",", "L"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "M", ",", "0", ",", "M", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "M", ",", "6", ",", "M", ",", "R"}], "}"}]}], "}"}]}],
   "]"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "ex3"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`ex3", "Print", "PrintUsage",
 CellTags->"Info3580720250-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"0", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "M", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"1", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "0", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"1", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "B", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"1", ",", "M"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "M", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "M", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"3", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"4", ",", "0", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"3", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "1", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"3", ",", "M"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"5", ",", "M", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"4", ",", "0"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"4", ",", "0", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"4", ",", "M"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "M", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"ex3", "[", 
          RowBox[{"5", ",", "M"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"6", ",", "M", ",", "R"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`ex3],
  Editable->False]], "Print",
 CellTags->"Info3580720250-4013843"]
}, Open  ]]
}, Open  ]],

Cell["\<\
To determine whether or not a string is in the language, we only have to \
apply the Turing machine to the string and check the exit state.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Turing", "[", 
  RowBox[{"ex3", ",", 
   RowBox[{"{", 
    RowBox[{
    "0", ",", "0", ",", "0", ",", "0", ",", "1", ",", "1", ",", "1", ",", 
     "1"}], "}"}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "M", ",", "M", ",", "M", ",", "M", ",", "M", ",", "M", ",", "M", ",", "M",
      ",", "B"}], "}"}], ",", "6"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
The fact that the machine halted in state 6, the final state, indicates that \
it recognizes the string. On the other hand,\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Turing", "[", 
  RowBox[{"ex3", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", "0"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"M", ",", "M", ",", "M", ",", "M", ",", "M", ",", "B"}], "}"}], 
   ",", "2"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
halted in state 2, indicating that the string is not in the language.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Adding Nonnegative Integers", "Subsubsection"],

Cell["\<\
Example 4 describes how to use Turing machines to perform addition.\
\>", "Text"],

Cell["The machine is described by the following tuples.", "Text"],

Cell[BoxData[
 RowBox[{"tuplesToIndexed", "[", 
  RowBox[{"adder", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1", ",", "B", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "\"\<*\>\"", ",", "3", ",", "B", ",", "R"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "2", ",", "B", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "1", ",", "2", ",", "1", ",", "R"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "\"\<*\>\"", ",", "3", ",", "1", ",", "R"}], "}"}]}], 
    "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We add two numbers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " by using the unary representation tape consisting of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "+", "1"}], TraditionalForm]]],
 " ones followed by an asterisk and then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "+", "1"}], TraditionalForm]]],
 " ones. We create a small function to create the tape given ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"unaryTape", "[", 
   RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"ConstantArray", "[", 
     RowBox[{"1", ",", 
      RowBox[{"{", 
       RowBox[{"a", "+", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "\"\<*\>\"", "}"}], ",", 
    RowBox[{"ConstantArray", "[", 
     RowBox[{"1", ",", 
      RowBox[{"{", 
       RowBox[{"b", "+", "1"}], "}"}]}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["The tape used to add 3 and 4 is shown below.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"unaryTape", "[", 
  RowBox[{"3", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "1", ",", "1", ",", "\<\"*\"\>", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Performing addition is accomplished by applying ",
 StyleBox["Turing", "Input"],
 " to the transition function and the tape."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Turing", "[", 
  RowBox[{"adder", ",", 
   RowBox[{"unaryTape", "[", 
    RowBox[{"3", ",", "4"}], "]"}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "B", ",", "B", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "1", ",", "1"}], "}"}], ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
You can see that this contains a string of 8 ones, indicating a result of 7.\
\>", "Text"],

Cell[TextData[{
 "Using the verbose form of ",
 StyleBox["Turing", "Input"],
 ", you can see how the Turing adder operates."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"verboseTuring", "[", 
  RowBox[{"adder", ",", 
   RowBox[{"unaryTape", "[", 
    RowBox[{"3", ",", "4"}], "]"}], ",", "0"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"\[Rule]1\"\>", ",", "1", ",", "1", ",", "1", 
     ",", "\<\"*\"\>", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
    "}"}], "\[InvisibleSpace]", "0"}],
  SequenceForm[{"\[Rule]1", 1, 1, 1, "*", 1, 1, 1, 1, 1}, 0],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "B", ",", "\<\"\[Rule]1\"\>", ",", "1", ",", "1", ",", "\<\"*\"\>", ",", 
     "1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], 
   "\[InvisibleSpace]", "1"}],
  SequenceForm[{$CellContext`B, "\[Rule]1", 1, 1, "*", 1, 1, 1, 1, 1}, 1],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "B", ",", "B", ",", "\<\"\[Rule]1\"\>", ",", "1", ",", "\<\"*\"\>", ",", 
     "1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], 
   "\[InvisibleSpace]", "2"}],
  SequenceForm[{$CellContext`B, $CellContext`B, "\[Rule]1", 1, "*", 1, 1, 1, 
    1, 1}, 2],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "B", ",", "B", ",", "1", ",", "\<\"\[Rule]1\"\>", ",", "\<\"*\"\>", ",", 
     "1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], 
   "\[InvisibleSpace]", "2"}],
  SequenceForm[{$CellContext`B, $CellContext`B, 1, "\[Rule]1", "*", 1, 1, 1, 
    1, 1}, 2],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "B", ",", "B", ",", "1", ",", "1", ",", "\<\"\[Rule]*\"\>", ",", "1", ",",
      "1", ",", "1", ",", "1", ",", "1"}], "}"}], "\[InvisibleSpace]", "2"}],
  SequenceForm[{$CellContext`B, $CellContext`B, 1, 1, "\[Rule]*", 1, 1, 1, 1, 
    1}, 2],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "B", ",", "B", ",", "1", ",", "1", ",", "1", ",", "\<\"\[Rule]1\"\>", ",",
      "1", ",", "1", ",", "1", ",", "1"}], "}"}], "\[InvisibleSpace]", "3"}],
  SequenceForm[{$CellContext`B, $CellContext`B, 1, 1, 1, "\[Rule]1", 1, 1, 1, 
    1}, 3],
  Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "B", ",", "B", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "1", ",", "1"}], "}"}], ",", "3"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 8", "Subsection",
 CellTags->"SolutionsCP8"],

Cell["\<\
Given the state table of a nondeterministic finite-state automaton and a \
string, decide whether this string is recognized by the automaton.\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " One solution to this problem, the solution used earlier in this chapter, \
is to find the deterministic automaton that recognizes the same language and \
use it to decide whether the string is recognized or not. This is what we \
have been doing when we apply ",
 StyleBox["findLanguage", "Input"],
 " to the result of ",
 StyleBox["makeDeterministic", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Here we will take a direct approach. For deterministic machines, we created \
two functions: ",
 StyleBox["extendedTransition", "Input"],
 " and ",
 StyleBox["recognizedQ", "Input"],
 ". The ",
 StyleBox["recognizedQ", "Input"],
 " function merely called ",
 StyleBox["extendedTransition", "Input"],
 " and checked whether the result was a final state or not. The ",
 StyleBox["extendedTransition", "Input"],
 " function took a state, an input string, and a transition table, and \
determined the state of the machine following the processing of the input."
}], "Text"],

Cell[TextData[{
 "Our approach for nondeterministic machines will be similar. We will create \
two functions: ",
 StyleBox["extendedTransitionND", "Input"],
 " and ",
 StyleBox["recognizedNDQ", "Input"],
 ". The main difference between the deterministic machines and \
nondeterministic machines is that with nondeterministic machines, given the \
initial state and an input, we do not know the next state. Instead, there is \
a set of possible states."
}], "Text"],

Cell[TextData[{
 StyleBox["extendedTransitionND", "Input"],
 " will therefore take a set of possible states, an input, and a transition \
table as its arguments. For each member of the input string, it will apply \
the transition table to each of the possible states, producing a new set of \
possible states. It will return the set of possible states after processing \
each element in the input string."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"extendedTransitionND", "[", 
   RowBox[{"states_", ",", "input_", ",", "transFunc_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"curStates", ",", "i", ",", "s", ",", "newStates"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"curStates", "=", "states"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "input", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"newStates", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{"newStates", "=", 
           RowBox[{"Union", "[", 
            RowBox[{"newStates", ",", 
             RowBox[{"transFunc", "[", 
              RowBox[{"s", ",", 
               RowBox[{"input", "[", 
                RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "]"}]}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"s", ",", "curStates"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"curStates", "=", "newStates"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", "curStates"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "A nondeterministic machine recognizes a string if the result of running the \
machine from the starting state with the input string results in a set of \
possible ending states that includes at least one final state. We write ",
 StyleBox["recognizedNDQ", "Input"],
 " to call ",
 StyleBox["extendedTransitionND", "Input"],
 " and check to see if the result intersects the set of final states."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"recognizedNDQ", "[", 
   RowBox[{"x_", ",", "transFunc_", ",", "init_", ",", "final_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "endStates", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"endStates", "=", 
      RowBox[{"extendedTransitionND", "[", 
       RowBox[{
        RowBox[{"{", "init", "}"}], ",", "x", ",", "transFunc"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Intersection", "[", 
       RowBox[{"endStates", ",", "final"}], "]"}], "\[NotEqual]", 
      RowBox[{"{", "}"}]}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "With ",
 StyleBox["recognizedNDQ", "Input"],
 " in hand, we can create ",
 StyleBox["findLanguageND", "Input"],
 ". This is effectively identical to ",
 StyleBox["findLanguage", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findLanguageND", "[", 
   RowBox[{"transFunc_", ",", "init_", ",", "final_", ",", "A_", ",", "n_"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"An", ",", "x", ",", "L"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"An", "=", 
      RowBox[{"kleene", "[", 
       RowBox[{"A", ",", "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"L", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"recognizedNDQ", "[", 
          RowBox[{"x", ",", "transFunc", ",", "init", ",", "final"}], "]"}], 
         ",", 
         RowBox[{"L", "=", 
          RowBox[{"Union", "[", 
           RowBox[{"L", ",", 
            RowBox[{"{", "x", "}"}]}], "]"}]}]}], "]"}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "An"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     "L"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying this function to the machine defined by transition function ",
 StyleBox["ctable", "Input"],
 ", starting state ",
 Cell[BoxData[
  FormBox[Cell["1"], TraditionalForm]]],
 ", final state ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "32", "}"}], TraditionalForm]]],
 ", and alphabet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], TraditionalForm]]],
 ", which was produced by ",
 StyleBox["catAutomata", "Input"],
 ", we see that the result is the same as when we applied ",
 StyleBox["findLanguage", "Input"],
 " and ",
 StyleBox["makeDeterministic", "Input"],
 " in Section 13.4."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLanguageND", "[", 
  RowBox[{"ctable", ",", "1", ",", 
   RowBox[{"{", "32", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", 
     "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0",
      ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1",
      ",", "0", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 1", "Subsection",
 CellTags->"SolutionsCE1"],

Cell[TextData[{
 "Solve the busy beaver problem for two states by testing all possible Turing \
machines with two states and alphabet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "B"}], "}"}], TraditionalForm]]],
 "."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " The busy beaver problem, described in the preface to Exercise 31 in \
Section 13.5, asks: what is the maximum number of ones that a Turing machine \
with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " states on the alphabet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "B"}], "}"}], TraditionalForm]]],
 " may print on an initially blank tape? This exercise asks us to solve the \
busy beaver problem with a brute force approach for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "2"}], TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
We will construct all possible Turing machines on 2 states with the given \
alphabet. For each possible Turing machine, we will allow it to run until \
either it halts, or until it has reached a predefined limit on the number of \
steps it is allowed. This later condition is important, since some of the \
possible machines will not halt on their own.\
\>", "Text"],

Cell[TextData[{
 "Generating all possible Turing machines on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "B"}], "}"}], TraditionalForm]]],
 " with two states is equivalent to finding all possible transition \
functions. The domain of a transition function is the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"S", "\[Times]", "I"}], "=", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], "\[Times]", 
     RowBox[{"{", 
      RowBox[{"1", ",", "B"}], "}"}]}]}], TraditionalForm]]],
 ". The codomain is the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", "2"}], "}"}], "\[Times]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "B"}], "}"}], "\[Times]", 
    RowBox[{"{", 
     RowBox[{"L", ",", "R"}], "}"}]}], TraditionalForm]]],
 ", where we use state 2 as a halting state, that is, a state which will \
cause the machine to halt."
}], "Text"],

Cell[TextData[{
 "We create the domain and codomain using the ",
 StyleBox[ButtonBox["Tuples",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tuples"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dom", "=", 
  RowBox[{"Tuples", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "B"}], "}"}]}], "}"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "B"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "B"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"codom", "=", 
  RowBox[{"Tuples", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "B"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"L", ",", "R"}], "}"}]}], "}"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "R"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "B", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "B", ",", "R"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "R"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "B", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "B", ",", "R"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "R"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "B", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "B", ",", "R"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now, each possible transition function is an assignment of each member of \
",
 Cell[BoxData[
  FormBox[
   StyleBox["dom",
    FontWeight->"Bold"], TraditionalForm]]],
 " to one of the members of ",
 Cell[BoxData[
  FormBox[
   StyleBox["codom",
    FontWeight->"Bold"], TraditionalForm]]],
 ". We can think of this as a member of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["codom",
     FontWeight->"Bold"], "4"], TraditionalForm]]],
 ", the Cartesian product of ",
 Cell[BoxData[
  FormBox[
   StyleBox["codom",
    FontWeight->"Bold"], TraditionalForm]]],
 " with itself four times. Each 4-tuple of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["codom", "Input"], "4"], TraditionalForm]]],
 " corresponds to the function that maps the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th member of ",
 Cell[BoxData[
  FormBox[
   StyleBox["dom",
    FontWeight->"Bold"], TraditionalForm]]],
 " to the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th element of the tuple. The function below accepts a member of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["codom", "Input"], "4"], TraditionalForm]]],
 " and produces the corresponding transition table."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"makeTable", ",", 
    RowBox[{"{", "HoldFirst", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeTable", "[", 
   RowBox[{"T_", ",", "t_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"j", ",", "d"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"For", "[", 
     RowBox[{
      RowBox[{"j", "=", "1"}], ",", 
      RowBox[{"j", "\[LessEqual]", "4"}], ",", 
      RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"d", "=", 
        RowBox[{"dom", "[", 
         RowBox[{"[", "j", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"T", "[", 
         RowBox[{
          RowBox[{"d", "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", 
          RowBox[{"d", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "=", 
        RowBox[{"t", "[", 
         RowBox[{"[", "j", "]"}], "]"}]}]}]}], "\[IndentingNewLine]", "]"}]}],
    "\[IndentingNewLine]", "]"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We now apply this function to each member of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["codom",
     FontWeight->"Bold"], "4"], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"codom4", "=", 
   RowBox[{"Tuples", "[", 
    RowBox[{"codom", ",", "4"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "codom4", "]"}]], "Input"],

Cell[BoxData["20736"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"i", "=", "1"}], ",", 
   RowBox[{"i", "\[LessEqual]", 
    RowBox[{"Length", "[", "codom4", "]"}]}], ",", 
   RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{"makeTable", "[", 
    RowBox[{
     RowBox[{"Symbol", "[", 
      RowBox[{"\"\<TF\>\"", "<>", 
       RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
     RowBox[{"codom4", "[", 
      RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Symbol",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Symbol"], "FunctionLink"],
 " function is used to convert a string into a symbol object. Here we use it \
to create variables ",
 StyleBox["TF1", "Input"],
 ", ",
 StyleBox["TF2", "Input"],
 ", ..., for the indexed variables that store the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"20", ",", "736"}], TraditionalForm]]],
 " transition tables."
}], "Text"],

Cell["\<\
The following function will count the number of ones that appear on a tape.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"count1s", "[", "L_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"count", "=", "0"}], ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "L", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"L", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "\[Equal]", "1"}], ",", 
         RowBox[{"count", "++"}]}], "]"}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "count"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We need to place a limit on the number of steps the Turing machine can take \
to avoid getting stuck in an infinite loop because of a machine that does not \
halt. For this, we create a version of ",
 StyleBox["Turing", "Input"],
 " specifically for this problem. It includes an extra argument for the limit \
on the number of steps and incorporates this limit into the main loop. We \
remove the argument for the initial tape and initial state, and instead set \
these to 0 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "B", "}"}], TraditionalForm]]],
 " in the function. Rather than returning the tape, this function will return \
the number of 1s appearing on the tape, assuming the machine halted. If it \
did not halt, we return -1."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"beaverTuring", "[", 
   RowBox[{"f_", ",", "maxsteps_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"pos", "=", "1"}], ",", 
      RowBox[{"state", "=", "0"}], ",", 
      RowBox[{"tape", "=", 
       RowBox[{"{", "B", "}"}]}], ",", "domain", ",", "y", ",", 
      RowBox[{"numsteps", "=", "0"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"domain", "=", 
      RowBox[{"getIndices", "[", "f", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"MemberQ", "[", 
         RowBox[{"domain", ",", 
          RowBox[{"{", 
           RowBox[{"state", ",", 
            RowBox[{"tape", "[", 
             RowBox[{"[", "pos", "]"}], "]"}]}], "}"}]}], "]"}], "&&", 
        RowBox[{"numsteps", "<", "maxsteps"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"y", "=", 
         RowBox[{"f", "[", 
          RowBox[{"state", ",", 
           RowBox[{"tape", "[", 
            RowBox[{"[", "pos", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"state", "=", 
         RowBox[{"y", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"tape", "[", 
          RowBox[{"[", "pos", "]"}], "]"}], "=", 
         RowBox[{"y", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Which", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"pos", "\[Equal]", "1"}], "&&", 
           RowBox[{
            RowBox[{"y", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "===", "L"}]}], ",", 
          RowBox[{"PrependTo", "[", 
           RowBox[{"tape", ",", "B"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"pos", "\[Equal]", 
            RowBox[{"Length", "[", "tape", "]"}]}], "&&", 
           RowBox[{
            RowBox[{"y", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "===", "R"}]}], ",", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"tape", ",", "B"}], "]"}], ";", 
           RowBox[{"pos", "++"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"y", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "===", "L"}], ",", 
          RowBox[{"pos", "--"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"y", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "===", "R"}], ",", 
          RowBox[{"pos", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"numsteps", "++"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"numsteps", "<", "maxsteps"}], ",", 
       RowBox[{"count1s", "[", "tape", "]"}], ",", 
       RowBox[{"-", "1"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now we apply ",
 StyleBox["beaverTuring", "Input"],
 " to each of the transition tables with a step limit of 100, keeping track \
of the number of 1s along the way. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"onesList", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", 
    RowBox[{"i", "\[LessEqual]", 
     RowBox[{"Length", "[", "codom4", "]"}]}], ",", 
    RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{"AppendTo", "[", 
     RowBox[{"onesList", ",", 
      RowBox[{"beaverTuring", "[", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<TF\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", "100"}], "]"}]}],
      "]"}]}], "\[IndentingNewLine]", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Max", "[", "onesList", "]"}]}], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using the ",
 StyleBox[ButtonBox["Tally",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tally"], "FunctionLink"],
 " function, we can see how many of the Turing machines produces tapes with \
each number of ones."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tally", "[", "onesList", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "10952"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "704"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4876"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "4184"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "16"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "4"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This shows us that ",
 Cell[BoxData[
  FormBox["4184", TraditionalForm]]],
 " of the machines halted with no ones on the tape, ",
 Cell[BoxData[
  FormBox["4", TraditionalForm]]],
 " machines halted with four ones, and ",
 Cell[BoxData[
  FormBox["10952", TraditionalForm]]],
 " of the machines failed to halt."
}], "Text"],

Cell[TextData[{
 "We can see the four machines that produced four ones as follows. The ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " function applied to a list and an expression will return the list of \
indices to the list at which the expression can be found."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{"onesList", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "7729", "}"}], ",", 
   RowBox[{"{", "7741", "}"}], ",", 
   RowBox[{"{", "9314", "}"}], ",", 
   RowBox[{"{", "9326", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["These are the transition functions for the four machines.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TF7729"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`TF7729", "Print", "PrintUsage",
 CellTags->"Info3580720268-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"TF7729", "[", 
          RowBox[{"0", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF7729", "[", 
          RowBox[{"0", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF7729", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF7729", "[", 
          RowBox[{"1", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "L"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`TF7729],
  Editable->False]], "Print",
 CellTags->"Info3580720268-4013843"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TF7741"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`TF7741", "Print", "PrintUsage",
 CellTags->"Info3580720268-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"TF7741", "[", 
          RowBox[{"0", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF7741", "[", 
          RowBox[{"0", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF7741", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF7741", "[", 
          RowBox[{"1", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "L"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`TF7741],
  Editable->False]], "Print",
 CellTags->"Info3580720268-4013843"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TF9314"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`TF9314", "Print", "PrintUsage",
 CellTags->"Info3580720268-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"TF9314", "[", 
          RowBox[{"0", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF9314", "[", 
          RowBox[{"0", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF9314", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF9314", "[", 
          RowBox[{"1", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "R"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`TF9314],
  Editable->False]], "Print",
 CellTags->"Info3580720268-4013843"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TF9327"}]], "Input"],

Cell[CellGroupData[{

Cell["Global`TF9327", "Print", "PrintUsage",
 CellTags->"Info3580720269-4013843"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"TF9327", "[", 
          RowBox[{"0", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF9327", "[", 
          RowBox[{"0", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "L"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF9327", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1", ",", "R"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"TF9327", "[", 
          RowBox[{"1", ",", "B"}], "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"0", ",", "B", ",", "L"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`TF9327],
  Editable->False]], "Print",
 CellTags->"Info3580720269-4013843"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "The busy beaver problem becomes very time consuming very quickly. Beyond ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "2"}], TraditionalForm]]],
 ", it is imperative to use more efficient approaches than was done here."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell["\<\
Construct the unit-delay machine described in Example 5 of Section 13.2.\
\>", "Item1Exercise"],

Cell[TextData[{
 "Construct a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for simulating the action of a Moore machine. (See the prelude to \
Exercise 21 in Section 13.2 for the definition of a Moore machine.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Develop ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for computing the union of two nondeterministic finite-state \
automata and for computing the Kleene closure of a nondeterministic \
finite-state machine, as described in the proof of Theorem 1 of Section 13.4 \
of the text."
}], "Item1Exercise"],

Cell[TextData[{
 "Develop ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for finding all the states of a finite-state machine that are \
reachable from a given state and for finding all transient states and sinks \
of the machine. (See Supplementary Exercise 16 for definitions.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Construct a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that computes the star height of a regular expression. (See \
Supplementary Exercise 11 for the definition of star height.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Construct a Turing machine that computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "-", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "\[GreaterEqual]", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 ". Test that this Turing machine produces the desired results for sample \
input values."
}], "Item1Exercise"],

Cell[TextData[{
 "Construct a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that simulates the action of a Turing machine that may move \
right, left, or not at all at each step."
}], "Item1Exercise"],

Cell[TextData[{
 "Construct a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that simulates the action of a Turing machine that may have more \
than one tape."
}], "Item1Exercise"],

Cell[TextData[{
 "Construct a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that simulates the action of a Turing machine with a \
two-dimensional tape. Represent a machine for multiplying integers and test \
it with your procedure."
}], "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{4, Automatic}, {Automatic, 4}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section2a", "Section2b", "Section3a",
                "Section3b", "Section3c", "Section3d", "Section4a", 
               "Section4b", "Section5a", "Section5b", "Section5c", 
               "SolutionsCE1", "SolutionsCP8", "Section1", "Section2", 
               "Section3", "Section4", "Section5", "Solutions", 
               "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section1"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section2"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section3"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section4"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section5"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Solutions"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Exercises"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section2", 
               "Section2a", "Section2b", "Section3", "Section3a", "Section3b",
                "Section3c", "Section3d", "Section4", "Section4a", 
               "Section4b", "Section5", "Section5a", "Section5b", "Section5c",
                "Solutions", "SolutionsCE1", "SolutionsCP8"}}] SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1231, 45, 69, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[2937, 101, 60, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Info3580720243-4013843"->{
  Cell[5640, 195, 90, 1, 40, "Print",
   CellTags->"Info3580720243-4013843"],
  Cell[5733, 198, 1377, 40, 115, "Print",
   CellTags->"Info3580720243-4013843"]},
 "Section1b"->{
  Cell[7672, 260, 60, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section2"->{
  Cell[23641, 741, 80, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[24314, 762, 61, 1, 32, "Subsection",
   CellTags->"Section2a"]},
 "Info3580720244-4013843"->{
  Cell[30119, 963, 83, 1, 40, "Print",
   CellTags->"Info3580720244-4013843"],
  Cell[30205, 966, 2560, 87, 301, "Print",
   CellTags->"Info3580720244-4013843"]},
 "Section2b"->{
  Cell[36343, 1168, 81, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section3"->{
  Cell[40174, 1314, 83, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[40481, 1326, 60, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[54054, 1784, 103, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section3c"->{
  Cell[57815, 1908, 93, 1, 26, "Subsection",
   CellTags->"Section3c"]},
 "Section3d"->{
  Cell[64277, 2132, 84, 1, 26, "Subsection",
   CellTags->"Section3d"]},
 "Section4"->{
  Cell[81801, 2681, 67, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[82323, 2699, 65, 1, 32, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[95104, 3151, 71, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Info3580720248-4013843"->{
  Cell[115742, 3871, 82, 1, 40, "Print",
   CellTags->"Info3580720248-4013843"],
  Cell[115827, 3874, 1911, 65, 239, "Print",
   CellTags->"Info3580720248-4013843"],
  Cell[118737, 3980, 82, 1, 40, "Print",
   CellTags->"Info3580720248-4013843"],
  Cell[118822, 3983, 3167, 113, 487, "Print",
   CellTags->"Info3580720248-4013843"],
  Cell[124005, 4163, 82, 1, 40, "Print",
   CellTags->"Info3580720248-4013843"],
  Cell[124090, 4166, 3260, 116, 487, "Print",
   CellTags->"Info3580720248-4013843"]},
 "Info3580720249-4013843"->{
  Cell[128865, 4336, 82, 1, 40, "Print",
   CellTags->"Info3580720249-4013843"],
  Cell[128950, 4339, 3271, 116, 487, "Print",
   CellTags->"Info3580720249-4013843"]},
 "Section5"->{
  Cell[142522, 4774, 62, 1, 35, "Section",
   CellTags->"Section5"]},
 "Section5a"->{
  Cell[143002, 4791, 88, 1, 32, "Subsection",
   CellTags->"Section5a"]},
 "Section5b"->{
  Cell[156091, 5244, 80, 1, 26, "Subsection",
   CellTags->"Section5b"]},
 "Info3580720250-4013843"->{
  Cell[159651, 5358, 78, 1, 40, "Print",
   CellTags->"Info3580720250-4013843"],
  Cell[159732, 5361, 2017, 66, 208, "Print",
   CellTags->"Info3580720250-4013843"],
  Cell[176257, 5873, 78, 1, 40, "Print",
   CellTags->"Info3580720250-4013843"],
  Cell[176338, 5876, 2997, 101, 363, "Print",
   CellTags->"Info3580720250-4013843"]},
 "Section5c"->{
  Cell[174025, 5800, 77, 1, 26, "Subsection",
   CellTags->"Section5c"]},
 "Solutions"->{
  Cell[186400, 6242, 115, 3, 35, "Section",
   CellTags->"Solutions"]},
 "SolutionsCP8"->{
  Cell[186540, 6249, 68, 1, 32, "Subsection",
   CellTags->"SolutionsCP8"]},
 "SolutionsCE1"->{
  Cell[194507, 6483, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE1"]},
 "Info3580720268-4013843"->{
  Cell[210304, 7008, 81, 1, 40, "Print",
   CellTags->"Info3580720268-4013843"],
  Cell[210388, 7011, 1444, 45, 115, "Print",
   CellTags->"Info3580720268-4013843"],
  Cell[211955, 7067, 81, 1, 40, "Print",
   CellTags->"Info3580720268-4013843"],
  Cell[212039, 7070, 1444, 45, 115, "Print",
   CellTags->"Info3580720268-4013843"],
  Cell[213606, 7126, 81, 1, 40, "Print",
   CellTags->"Info3580720268-4013843"],
  Cell[213690, 7129, 1444, 45, 115, "Print",
   CellTags->"Info3580720268-4013843"]},
 "Info3580720269-4013843"->{
  Cell[215257, 7185, 81, 1, 40, "Print",
   CellTags->"Info3580720269-4013843"],
  Cell[215341, 7188, 1444, 45, 115, "Print",
   CellTags->"Info3580720269-4013843"]},
 "Exercises"->{
  Cell[217115, 7249, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 236374, 7658},
 {"Section1a", 236456, 7661},
 {"Info3580720243-4013843", 236556, 7664},
 {"Section1b", 236733, 7669},
 {"Section2", 236819, 7672},
 {"Section2a", 236903, 7675},
 {"Info3580720244-4013843", 237004, 7678},
 {"Section2b", 237183, 7683},
 {"Section3", 237271, 7686},
 {"Section3a", 237356, 7689},
 {"Section3b", 237445, 7692},
 {"Section3c", 237535, 7695},
 {"Section3d", 237624, 7698},
 {"Section4", 237712, 7701},
 {"Section4a", 237797, 7704},
 {"Section4b", 237886, 7707},
 {"Info3580720248-4013843", 237988, 7710},
 {"Info3580720249-4013843", 238518, 7723},
 {"Section5", 238701, 7728},
 {"Section5a", 238787, 7731},
 {"Section5b", 238877, 7734},
 {"Info3580720250-4013843", 238980, 7737},
 {"Section5c", 239330, 7746},
 {"Solutions", 239420, 7749},
 {"SolutionsCP8", 239511, 7752},
 {"SolutionsCE1", 239607, 7755},
 {"Info3580720268-4013843", 239713, 7758},
 {"Info3580720269-4013843", 240241, 7771},
 {"Exercises", 240424, 7776}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 94, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 13}}],
Cell[CellGroupData[{
Cell[698, 27, 31, 0, 55, "Section"],
Cell[732, 29, 462, 11, 50, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1231, 45, 69, 1, 55, "Section",
 CellTags->"Section1"],
Cell[1303, 48, 443, 14, 35, "Text"],
Cell[1749, 64, 409, 10, 35, "Text"],
Cell[2161, 76, 432, 12, 35, "Text"],
Cell[2596, 90, 316, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[2937, 101, 60, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[3000, 104, 164, 6, 20, "Text"],
Cell[3167, 112, 200, 4, 35, "Text"],
Cell[3370, 118, 362, 6, 50, "Text"],
Cell[3735, 126, 1010, 37, 50, "Text"],
Cell[4748, 165, 56, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[4829, 169, 786, 22, 93, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[5640, 195, 90, 1, 40, "Print",
 CellTags->"Info3580720243-4013843"],
Cell[5733, 198, 1377, 40, 115, "Print",
 CellTags->"Info3580720243-4013843"]
}, Open  ]]
}, Open  ]],
Cell[7137, 242, 498, 13, 52, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[7672, 260, 60, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[7735, 263, 421, 12, 37, "Text"],
Cell[8159, 277, 513, 15, 37, "Text"],
Cell[8675, 294, 686, 19, 69, "Text"],
Cell[9364, 315, 715, 21, 53, "Text"],
Cell[CellGroupData[{
Cell[10104, 340, 110, 2, 29, "Input"],
Cell[10217, 344, 171, 6, 28, "Output"]
}, Open  ]],
Cell[10403, 353, 393, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[10821, 365, 115, 2, 29, "Input"],
Cell[10939, 369, 171, 6, 28, "Output"]
}, Open  ]],
Cell[11125, 378, 99, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[11249, 384, 110, 2, 29, "Input"],
Cell[11362, 388, 45, 1, 28, "Output"]
}, Open  ]],
Cell[11422, 392, 308, 7, 37, "Text"],
Cell[11733, 401, 226, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[11984, 409, 97, 2, 29, "Input",
 InitializationCell->True],
Cell[12084, 413, 41, 0, 28, "Output"]
}, Open  ]],
Cell[12140, 416, 117, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[12282, 423, 106, 2, 29, "Input"],
Cell[12391, 427, 45, 1, 28, "Output"]
}, Open  ]],
Cell[12451, 431, 160, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[12636, 438, 106, 2, 29, "Input"],
Cell[12745, 442, 171, 6, 28, "Output"]
}, Open  ]],
Cell[12931, 451, 202, 4, 20, "Text"],
Cell[CellGroupData[{
Cell[13158, 459, 76, 1, 29, "Input"],
Cell[13237, 462, 112, 3, 28, "Output"]
}, Open  ]],
Cell[13364, 468, 741, 14, 66, "Text"],
Cell[14108, 484, 788, 17, 68, "Text"],
Cell[CellGroupData[{
Cell[14921, 505, 176, 4, 29, "Input"],
Cell[15100, 511, 42, 0, 29, "Output"]
}, Open  ]],
Cell[15157, 514, 61, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[15243, 518, 529, 15, 77, "Input"],
Cell[CellGroupData[{
Cell[15797, 537, 41, 0, 21, "Print"],
Cell[15841, 539, 41, 0, 21, "Print"],
Cell[15885, 541, 41, 0, 21, "Print"],
Cell[15929, 543, 41, 0, 21, "Print"],
Cell[15973, 545, 40, 0, 21, "Print"],
Cell[16016, 547, 40, 0, 21, "Print"]
}, Open  ]]
}, Open  ]],
Cell[16083, 551, 626, 17, 37, "Text"],
Cell[16712, 570, 899, 20, 83, "Text"],
Cell[CellGroupData[{
Cell[17636, 594, 214, 6, 29, "Input"],
Cell[17853, 602, 45, 1, 28, "Output"]
}, Open  ]],
Cell[17913, 606, 37, 0, 20, "Text"],
Cell[17953, 608, 3112, 75, 361, "Input",
 InitializationCell->True],
Cell[21068, 685, 112, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[21205, 692, 391, 9, 45, "Input"],
Cell[21599, 703, 1981, 31, 179, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[23641, 741, 80, 1, 35, "Section",
 CellTags->"Section2"],
Cell[23724, 744, 565, 14, 50, "Text"],
Cell[CellGroupData[{
Cell[24314, 762, 61, 1, 32, "Subsection",
 CellTags->"Section2a"],
Cell[24378, 765, 606, 22, 35, "Text"],
Cell[24987, 789, 541, 12, 50, "Text"],
Cell[CellGroupData[{
Cell[25553, 805, 39, 0, 18, "Subsubsection"],
Cell[25595, 807, 173, 6, 20, "Text"],
Cell[25771, 815, 567, 17, 50, "Text"],
Cell[26341, 834, 544, 18, 35, "Text"],
Cell[26888, 854, 1118, 33, 69, "Text"],
Cell[28009, 889, 171, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[28205, 896, 1889, 63, 179, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[30119, 963, 83, 1, 40, "Print",
 CellTags->"Info3580720244-4013843"],
Cell[30205, 966, 2560, 87, 301, "Print",
 CellTags->"Info3580720244-4013843"]
}, Open  ]]
}, Open  ]],
Cell[32792, 1057, 126, 3, 20, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32955, 1065, 54, 0, 18, "Subsubsection"],
Cell[33012, 1067, 206, 4, 35, "Text"],
Cell[33221, 1073, 529, 10, 67, "Text"],
Cell[33753, 1085, 572, 13, 52, "Text"],
Cell[34328, 1100, 37, 0, 20, "Text"],
Cell[34368, 1102, 1424, 38, 172, "Input",
 InitializationCell->True],
Cell[35795, 1142, 151, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[35971, 1151, 192, 5, 29, "Input"],
Cell[36166, 1158, 116, 3, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[36343, 1168, 81, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[36427, 1171, 235, 4, 35, "Text"],
Cell[36665, 1177, 654, 26, 35, "Text"],
Cell[37322, 1205, 500, 18, 35, "Text"],
Cell[37825, 1225, 99, 2, 20, "Text"],
Cell[37927, 1229, 1524, 49, 134, "Input",
 InitializationCell->True],
Cell[39454, 1280, 161, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[39640, 1288, 215, 6, 29, "Input"],
Cell[39858, 1296, 93, 2, 28, "Output"]
}, Open  ]],
Cell[39966, 1301, 159, 7, 20, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[40174, 1314, 83, 1, 35, "Section",
 CellTags->"Section3"],
Cell[40260, 1317, 196, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[40481, 1326, 60, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[40544, 1329, 236, 4, 35, "Text"],
Cell[40783, 1335, 637, 14, 51, "Text"],
Cell[41423, 1351, 1979, 53, 135, "Input",
 InitializationCell->True],
Cell[43405, 1406, 536, 15, 37, "Text"],
Cell[43944, 1423, 136, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[44105, 1430, 178, 6, 28, "Input",
 InitializationCell->True],
Cell[44286, 1438, 122, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44445, 1447, 249, 8, 28, "Input",
 InitializationCell->True],
Cell[44697, 1457, 191, 6, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44925, 1468, 90, 2, 28, "Input"],
Cell[45018, 1472, 500, 15, 28, "Output"]
}, Open  ]],
Cell[45533, 1490, 1369, 54, 45, "Text"],
Cell[46905, 1546, 215, 5, 20, "Text"],
Cell[47123, 1553, 465, 16, 29, "Input",
 InitializationCell->True],
Cell[47591, 1571, 301, 12, 23, "Text"],
Cell[CellGroupData[{
Cell[47917, 1587, 86, 2, 28, "Input"],
Cell[48006, 1591, 2828, 77, 119, "Output"]
}, Open  ]],
Cell[50849, 1671, 579, 20, 39, "Text"],
Cell[51431, 1693, 1291, 37, 150, "Input",
 InitializationCell->True],
Cell[52725, 1732, 183, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[52933, 1743, 131, 4, 28, "Input"],
Cell[53067, 1749, 938, 29, 44, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[54054, 1784, 103, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[54160, 1787, 204, 4, 35, "Text"],
Cell[54367, 1793, 270, 5, 35, "Text"],
Cell[54640, 1800, 204, 6, 20, "Text"],
Cell[54847, 1808, 557, 20, 74, "Input",
 InitializationCell->True],
Cell[55407, 1830, 379, 7, 51, "Text"],
Cell[55789, 1839, 364, 6, 50, "Text"],
Cell[56156, 1847, 884, 24, 107, "Input",
 InitializationCell->True],
Cell[57043, 1873, 472, 18, 20, "Text"],
Cell[CellGroupData[{
Cell[57540, 1895, 195, 5, 28, "Input"],
Cell[57738, 1902, 28, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[57815, 1908, 93, 1, 26, "Subsection",
 CellTags->"Section3c"],
Cell[57911, 1911, 313, 9, 35, "Text"],
Cell[58227, 1922, 570, 17, 37, "Text"],
Cell[58800, 1941, 296, 8, 36, "Text"],
Cell[59099, 1951, 577, 15, 75, "Input",
 InitializationCell->True],
Cell[59679, 1968, 273, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[59977, 1981, 222, 6, 29, "Input"],
Cell[60202, 1989, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60270, 1994, 212, 6, 29, "Input"],
Cell[60485, 2002, 32, 0, 28, "Output"]
}, Open  ]],
Cell[60532, 2005, 188, 5, 21, "Text"],
Cell[60723, 2012, 505, 15, 38, "Text"],
Cell[61231, 2029, 302, 11, 23, "Text"],
Cell[61536, 2042, 985, 27, 107, "Input",
 InitializationCell->True],
Cell[62524, 2071, 427, 14, 38, "Text"],
Cell[CellGroupData[{
Cell[62976, 2089, 200, 5, 29, "Input"],
Cell[63179, 2096, 1049, 30, 59, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[64277, 2132, 84, 1, 26, "Subsection",
 CellTags->"Section3d"],
Cell[64364, 2135, 246, 4, 35, "Text"],
Cell[64613, 2141, 326, 5, 50, "Text"],
Cell[64942, 2148, 253, 4, 35, "Text"],
Cell[65198, 2154, 162, 3, 20, "Text"],
Cell[65363, 2159, 1583, 52, 164, "Input",
 InitializationCell->True],
Cell[66949, 2213, 252, 4, 35, "Text"],
Cell[67204, 2219, 405, 6, 50, "Text"],
Cell[67612, 2227, 725, 23, 50, "Text"],
Cell[68340, 2252, 619, 19, 50, "Text"],
Cell[68962, 2273, 366, 6, 50, "Text"],
Cell[69331, 2281, 346, 6, 50, "Text"],
Cell[69680, 2289, 3238, 86, 304, "Input",
 InitializationCell->True],
Cell[72921, 2377, 100, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[73046, 2383, 330, 9, 28, "Input"],
Cell[73379, 2394, 387, 13, 28, "Output"]
}, Open  ]],
Cell[73781, 2410, 298, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[74104, 2424, 72, 1, 28, "Input"],
Cell[74179, 2427, 4017, 138, 494, "Output"]
}, Open  ]],
Cell[78211, 2568, 90, 2, 20, "Text"],
Cell[78304, 2572, 111, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[78440, 2580, 194, 4, 29, "Input"],
Cell[78637, 2586, 2915, 82, 119, "Output"]
}, Open  ]],
Cell[81567, 2671, 185, 4, 35, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[81801, 2681, 67, 1, 35, "Section",
 CellTags->"Section4"],
Cell[81871, 2684, 427, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[82323, 2699, 65, 1, 32, "Subsection",
 CellTags->"Section4a"],
Cell[82391, 2702, 1541, 42, 84, "Text"],
Cell[83935, 2746, 257, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[84217, 2755, 151, 3, 29, "Input"],
Cell[84371, 2760, 31, 0, 28, "Output"]
}, Open  ]],
Cell[84417, 2763, 201, 4, 20, "Text"],
Cell[CellGroupData[{
Cell[84643, 2771, 39, 0, 18, "Subsubsection"],
Cell[84685, 2773, 112, 3, 20, "Text"],
Cell[84800, 2778, 157, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[84982, 2785, 158, 3, 29, "Input"],
Cell[85143, 2790, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85211, 2795, 151, 3, 29, "Input"],
Cell[85365, 2800, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85433, 2805, 157, 3, 29, "Input"],
Cell[85593, 2810, 32, 0, 28, "Output"]
}, Open  ]],
Cell[85640, 2813, 233, 4, 20, "Text"],
Cell[CellGroupData[{
Cell[85898, 2821, 170, 3, 29, "Input"],
Cell[86071, 2826, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86139, 2831, 161, 3, 29, "Input"],
Cell[86303, 2836, 32, 0, 28, "Output"]
}, Open  ]],
Cell[86350, 2839, 697, 12, 50, "Text"],
Cell[CellGroupData[{
Cell[87072, 2855, 158, 3, 29, "Input"],
Cell[87233, 2860, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87301, 2865, 153, 3, 29, "Input"],
Cell[87457, 2870, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87525, 2875, 154, 3, 29, "Input"],
Cell[87682, 2880, 32, 0, 28, "Output"]
}, Open  ]],
Cell[87729, 2883, 308, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[88062, 2892, 157, 3, 29, "Input"],
Cell[88222, 2897, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88290, 2902, 158, 3, 29, "Input"],
Cell[88451, 2907, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88519, 2912, 159, 3, 29, "Input"],
Cell[88681, 2917, 32, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[88762, 2923, 30, 0, 18, "Subsubsection"],
Cell[88795, 2925, 419, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[89239, 2936, 163, 4, 29, "Input"],
Cell[89405, 2942, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89473, 2947, 164, 4, 29, "Input"],
Cell[89640, 2953, 32, 0, 28, "Output"]
}, Open  ]],
Cell[89687, 2956, 279, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[89991, 2965, 173, 4, 29, "Input"],
Cell[90167, 2971, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90235, 2976, 171, 4, 29, "Input"],
Cell[90409, 2982, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90477, 2987, 171, 4, 29, "Input"],
Cell[90651, 2993, 32, 0, 28, "Output"]
}, Open  ]],
Cell[90698, 2996, 322, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[91045, 3005, 162, 4, 29, "Input"],
Cell[91210, 3011, 31, 0, 28, "Output"]
}, Open  ]],
Cell[91256, 3014, 94, 2, 20, "Text"],
Cell[91353, 3018, 240, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[91618, 3026, 166, 4, 29, "Input"],
Cell[91787, 3032, 31, 0, 28, "Output"]
}, Open  ]],
Cell[91833, 3035, 330, 5, 50, "Text"],
Cell[CellGroupData[{
Cell[92188, 3044, 165, 4, 29, "Input"],
Cell[92356, 3050, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[92424, 3055, 166, 4, 29, "Input"],
Cell[92593, 3061, 32, 0, 28, "Output"]
}, Open  ]],
Cell[92640, 3064, 606, 10, 50, "Text"],
Cell[CellGroupData[{
Cell[93271, 3078, 171, 4, 29, "Input"],
Cell[93445, 3084, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93513, 3089, 171, 4, 29, "Input"],
Cell[93687, 3095, 32, 0, 28, "Output"]
}, Open  ]],
Cell[93734, 3098, 300, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[94059, 3107, 157, 3, 29, "Input"],
Cell[94219, 3112, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94287, 3117, 156, 3, 29, "Input"],
Cell[94446, 3122, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94515, 3127, 162, 3, 29, "Input"],
Cell[94680, 3132, 32, 0, 28, "Output"]
}, Open  ]],
Cell[94727, 3135, 328, 10, 35, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[95104, 3151, 71, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[95178, 3154, 164, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[95367, 3161, 37, 0, 18, "Subsubsection"],
Cell[95407, 3163, 93, 2, 20, "Text"],
Cell[95503, 3167, 217, 4, 35, "Text"],
Cell[95723, 3173, 590, 14, 50, "Text"],
Cell[96316, 3189, 106, 3, 20, "Text"],
Cell[96425, 3194, 1216, 41, 134, "Input",
 InitializationCell->True],
Cell[97644, 3237, 388, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[98057, 3253, 355, 10, 28, "Input",
 InitializationCell->True],
Cell[98415, 3265, 234, 8, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[98686, 3278, 185, 4, 29, "Input"],
Cell[98874, 3284, 1108, 32, 59, "Output"]
}, Open  ]],
Cell[99997, 3319, 215, 4, 20, "Text"],
Cell[100215, 3325, 136, 3, 20, "Text"],
Cell[100354, 3330, 1211, 41, 134, "Input",
 InitializationCell->True],
Cell[101568, 3373, 114, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[101707, 3380, 319, 9, 28, "Input",
 InitializationCell->True],
Cell[102029, 3391, 146, 5, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[102212, 3401, 185, 4, 29, "Input"],
Cell[102400, 3407, 107, 3, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[102556, 3416, 51, 0, 18, "Subsubsection"],
Cell[102610, 3418, 300, 5, 35, "Text"],
Cell[102913, 3425, 152, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[103090, 3432, 153, 3, 16, "Item1Numbered"],
Cell[103246, 3437, 130, 3, 16, "Item1Numbered"],
Cell[103379, 3442, 127, 3, 16, "Item1Numbered"],
Cell[103509, 3447, 155, 3, 16, "Item1Numbered"],
Cell[103667, 3452, 109, 2, 16, "Item1Numbered"],
Cell[103779, 3456, 204, 4, 29, "Item1Numbered"],
Cell[103986, 3462, 235, 4, 29, "Item1Numbered"]
}, Open  ]],
Cell[104236, 3469, 413, 6, 50, "Text"],
Cell[104652, 3477, 552, 20, 35, "Text"],
Cell[105207, 3499, 381, 8, 51, "Text"],
Cell[105591, 3509, 89, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[105705, 3515, 138, 4, 28, "Input",
 InitializationCell->True],
Cell[105846, 3521, 83, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105966, 3528, 104, 3, 28, "Input",
 InitializationCell->True],
Cell[106073, 3533, 50, 1, 28, "Output"]
}, Open  ]],
Cell[106138, 3537, 292, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[106455, 3550, 185, 7, 29, "Input"],
Cell[106643, 3559, 51, 1, 28, "Output"]
}, Open  ]],
Cell[106709, 3563, 538, 8, 65, "Text"],
Cell[107250, 3573, 277, 7, 36, "Text"],
Cell[107530, 3582, 64, 1, 28, "Input"],
Cell[107597, 3585, 354, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[107976, 3598, 68, 1, 28, "Input"],
Cell[108047, 3601, 1433, 42, 74, "Output"]
}, Open  ]],
Cell[109495, 3646, 390, 14, 21, "Text"],
Cell[CellGroupData[{
Cell[109910, 3664, 205, 6, 28, "Input",
 InitializationCell->True],
Cell[110118, 3672, 933, 26, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[111088, 3703, 247, 7, 29, "Input",
 InitializationCell->True],
Cell[111338, 3712, 853, 26, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112228, 3743, 169, 5, 28, "Input",
 InitializationCell->True],
Cell[112400, 3750, 525, 18, 28, "Output"]
}, Open  ]],
Cell[112940, 3771, 137, 4, 21, "Text"],
Cell[113080, 3777, 850, 23, 92, "Input",
 InitializationCell->True],
Cell[113933, 3802, 951, 31, 68, "Text"],
Cell[114887, 3835, 651, 21, 45, "Input",
 InitializationCell->True],
Cell[115541, 3858, 101, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[115667, 3866, 50, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[115742, 3871, 82, 1, 40, "Print",
 CellTags->"Info3580720248-4013843"],
Cell[115827, 3874, 1911, 65, 239, "Print",
 CellTags->"Info3580720248-4013843"]
}, Open  ]]
}, Open  ]],
Cell[117765, 3943, 98, 2, 20, "Text"],
Cell[117866, 3947, 771, 24, 61, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[118662, 3975, 50, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[118737, 3980, 82, 1, 40, "Print",
 CellTags->"Info3580720248-4013843"],
Cell[118822, 3983, 3167, 113, 487, "Print",
 CellTags->"Info3580720248-4013843"]
}, Open  ]]
}, Open  ]],
Cell[122016, 4100, 288, 5, 35, "Text"],
Cell[122307, 4107, 429, 9, 52, "Text"],
Cell[122739, 4118, 1093, 34, 59, "Input",
 InitializationCell->True],
Cell[123835, 4154, 70, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[123930, 4158, 50, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[124005, 4163, 82, 1, 40, "Print",
 CellTags->"Info3580720248-4013843"],
Cell[124090, 4166, 3260, 116, 487, "Print",
 CellTags->"Info3580720248-4013843"]
}, Open  ]]
}, Open  ]],
Cell[127377, 4286, 360, 6, 50, "Text"],
Cell[127740, 4294, 982, 31, 76, "Input",
 InitializationCell->True],
Cell[128725, 4327, 40, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[128790, 4331, 50, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[128865, 4336, 82, 1, 40, "Print",
 CellTags->"Info3580720249-4013843"],
Cell[128950, 4339, 3271, 116, 487, "Print",
 CellTags->"Info3580720249-4013843"]
}, Open  ]]
}, Open  ]],
Cell[132248, 4459, 736, 24, 50, "Text"],
Cell[132987, 4485, 874, 19, 80, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[133898, 4509, 53, 0, 18, "Subsubsection"],
Cell[133954, 4511, 73, 0, 20, "Text"],
Cell[134030, 4513, 5908, 166, 432, "Input",
 InitializationCell->True],
Cell[139941, 4681, 298, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[140264, 4692, 346, 9, 28, "Input",
 InitializationCell->True],
Cell[140613, 4703, 98, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[140748, 4711, 309, 9, 28, "Input",
 InitializationCell->True],
Cell[141060, 4722, 147, 5, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[141244, 4732, 186, 4, 29, "Input"],
Cell[141433, 4738, 1016, 28, 59, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[142522, 4774, 62, 1, 35, "Section",
 CellTags->"Section5"],
Cell[142587, 4777, 390, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[143002, 4791, 88, 1, 32, "Subsection",
 CellTags->"Section5a"],
Cell[143093, 4794, 1535, 55, 35, "Text"],
Cell[144631, 4851, 1192, 36, 68, "Text"],
Cell[CellGroupData[{
Cell[145848, 4891, 1265, 40, 45, "Input",
 InitializationCell->True],
Cell[147116, 4933, 1167, 38, 44, "Output"]
}, Open  ]],
Cell[148298, 4974, 598, 11, 66, "Text"],
Cell[148899, 4987, 309, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[149233, 4998, 294, 9, 29, "Input",
 InitializationCell->True],
Cell[149530, 5009, 228, 7, 28, "Output"]
}, Open  ]],
Cell[149773, 5019, 216, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[150014, 5029, 112, 2, 29, "Input"],
Cell[150129, 5033, 287, 9, 28, "Output"]
}, Open  ]],
Cell[150431, 5045, 112, 3, 20, "Text"],
Cell[150546, 5050, 332, 12, 22, "Text"],
Cell[150881, 5064, 607, 16, 50, "Text"],
Cell[151491, 5082, 206, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[151722, 5090, 369, 11, 29, "Input"],
Cell[152094, 5103, 287, 9, 28, "Output"]
}, Open  ]],
Cell[152396, 5115, 106, 3, 20, "Text"],
Cell[152505, 5120, 97, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[152627, 5126, 122, 2, 29, "Input"],
Cell[152752, 5130, 1385, 44, 59, "Output"]
}, Open  ]],
Cell[154152, 5177, 194, 4, 35, "Text"],
Cell[154349, 5183, 474, 11, 36, "Text"],
Cell[CellGroupData[{
Cell[154848, 5198, 810, 25, 91, "Input"],
Cell[155661, 5225, 287, 9, 28, "Output"]
}, Open  ]],
Cell[155963, 5237, 91, 2, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[156091, 5244, 80, 1, 26, "Subsection",
 CellTags->"Section5b"],
Cell[156174, 5247, 541, 14, 52, "Text"],
Cell[156718, 5263, 118, 3, 28, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[156861, 5270, 45, 0, 18, "Subsubsection"],
Cell[156909, 5272, 595, 17, 50, "Text"],
Cell[157507, 5291, 212, 4, 35, "Text"],
Cell[157722, 5297, 899, 26, 90, "Input",
 InitializationCell->True],
Cell[158624, 5325, 152, 3, 20, "Text"],
Cell[158779, 5330, 776, 19, 45, "Input"],
Cell[CellGroupData[{
Cell[159580, 5353, 46, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[159651, 5358, 78, 1, 40, "Print",
 CellTags->"Info3580720250-4013843"],
Cell[159732, 5361, 2017, 66, 208, "Print",
 CellTags->"Info3580720250-4013843"]
}, Open  ]]
}, Open  ]],
Cell[161776, 5431, 270, 9, 36, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[162083, 5445, 52, 0, 18, "Subsubsection"],
Cell[162138, 5447, 278, 5, 35, "Text"],
Cell[162419, 5454, 605, 14, 68, "Text"],
Cell[163027, 5470, 363, 8, 36, "Text"],
Cell[163393, 5480, 1071, 23, 98, "Text"],
Cell[164467, 5505, 37, 0, 20, "Text"],
Cell[164507, 5507, 2645, 70, 221, "Input",
 InitializationCell->True],
Cell[167155, 5579, 118, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[167298, 5586, 187, 5, 29, "Input"],
Cell[167488, 5593, 166, 5, 28, "Output"]
}, Open  ]],
Cell[167669, 5601, 93, 2, 20, "Text"],
Cell[167765, 5605, 233, 5, 36, "Text"],
Cell[168001, 5612, 3648, 93, 333, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[171674, 5709, 194, 5, 29, "Input"],
Cell[CellGroupData[{
Cell[171893, 5718, 264, 7, 21, "Print"],
Cell[172160, 5727, 268, 8, 21, "Print"],
Cell[172431, 5737, 268, 8, 21, "Print"],
Cell[172702, 5747, 268, 8, 21, "Print"],
Cell[172973, 5757, 268, 8, 21, "Print"],
Cell[173244, 5767, 268, 8, 21, "Print"],
Cell[173515, 5777, 268, 8, 21, "Print"]
}, Open  ]],
Cell[173798, 5788, 166, 5, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[174025, 5800, 77, 1, 26, "Subsection",
 CellTags->"Section5c"],
Cell[174105, 5803, 146, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[174276, 5810, 41, 0, 18, "Subsubsection"],
Cell[174320, 5812, 306, 11, 22, "Text"],
Cell[174629, 5825, 135, 3, 20, "Text"],
Cell[174767, 5830, 1394, 34, 76, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[176186, 5868, 46, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[176257, 5873, 78, 1, 40, "Print",
 CellTags->"Info3580720250-4013843"],
Cell[176338, 5876, 2997, 101, 363, "Print",
 CellTags->"Info3580720250-4013843"]
}, Open  ]]
}, Open  ]],
Cell[179362, 5981, 163, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[179550, 5988, 214, 6, 29, "Input"],
Cell[179767, 5996, 203, 6, 28, "Output"]
}, Open  ]],
Cell[179985, 6005, 147, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[180157, 6012, 176, 5, 29, "Input"],
Cell[180336, 6019, 166, 5, 28, "Output"]
}, Open  ]],
Cell[180517, 6027, 93, 2, 20, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[180647, 6034, 52, 0, 18, "Subsubsection"],
Cell[180702, 6036, 91, 2, 20, "Text"],
Cell[180796, 6040, 65, 0, 20, "Text"],
Cell[180864, 6042, 643, 17, 45, "Input",
 InitializationCell->True],
Cell[181510, 6061, 591, 22, 35, "Text"],
Cell[182104, 6085, 483, 15, 29, "Input",
 InitializationCell->True],
Cell[182590, 6102, 60, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[182675, 6106, 85, 2, 29, "Input"],
Cell[182763, 6110, 168, 4, 28, "Output"]
}, Open  ]],
Cell[182946, 6117, 155, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[183126, 6125, 158, 4, 29, "Input"],
Cell[183287, 6131, 213, 6, 28, "Output"]
}, Open  ]],
Cell[183515, 6140, 100, 2, 20, "Text"],
Cell[183618, 6144, 136, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[183779, 6152, 165, 4, 29, "Input"],
Cell[CellGroupData[{
Cell[183969, 6160, 331, 8, 21, "Print"],
Cell[184303, 6170, 348, 9, 21, "Print"],
Cell[184654, 6181, 366, 10, 21, "Print"],
Cell[185023, 6193, 366, 10, 21, "Print"],
Cell[185392, 6205, 352, 9, 21, "Print"],
Cell[185747, 6216, 352, 9, 21, "Print"]
}, Open  ]],
Cell[186114, 6228, 213, 6, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[186400, 6242, 115, 3, 35, "Section",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[186540, 6249, 68, 1, 32, "Subsection",
 CellTags->"SolutionsCP8"],
Cell[186611, 6252, 168, 3, 55, "Problem"],
Cell[186782, 6257, 445, 11, 51, "Text"],
Cell[187230, 6270, 587, 14, 68, "Text"],
Cell[187820, 6286, 464, 10, 52, "Text"],
Cell[188287, 6298, 417, 7, 51, "Text"],
Cell[188707, 6307, 1427, 36, 184, "Input",
 InitializationCell->True],
Cell[190137, 6345, 424, 8, 51, "Text"],
Cell[190564, 6355, 674, 17, 75, "Input",
 InitializationCell->True],
Cell[191241, 6374, 222, 8, 21, "Text"],
Cell[191466, 6384, 1090, 30, 122, "Input",
 InitializationCell->True],
Cell[192559, 6416, 655, 22, 53, "Text"],
Cell[CellGroupData[{
Cell[193239, 6442, 200, 5, 29, "Input"],
Cell[193442, 6449, 1016, 28, 59, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[194507, 6483, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE1"],
Cell[194590, 6486, 255, 8, 42, "Problem"],
Cell[194848, 6496, 624, 19, 50, "Text"],
Cell[195475, 6517, 376, 6, 50, "Text"],
Cell[195854, 6525, 944, 29, 50, "Text"],
Cell[196801, 6556, 194, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[197020, 6566, 266, 9, 29, "Input",
 InitializationCell->True],
Cell[197289, 6577, 289, 10, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[197615, 6592, 341, 11, 29, "Input",
 InitializationCell->True],
Cell[197959, 6605, 881, 26, 44, "Output"]
}, Open  ]],
Cell[198855, 6634, 1250, 45, 71, "Text"],
Cell[200108, 6681, 1097, 31, 124, "Input",
 InitializationCell->True],
Cell[201208, 6714, 201, 8, 23, "Text"],
Cell[201412, 6724, 162, 5, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[201599, 6733, 64, 1, 29, "Input"],
Cell[201666, 6736, 32, 0, 28, "Output"]
}, Open  ]],
Cell[201713, 6739, 505, 14, 61, "Input"],
Cell[202221, 6755, 456, 15, 36, "Text"],
Cell[202680, 6772, 99, 2, 20, "Text"],
Cell[202782, 6776, 818, 24, 106, "Input",
 InitializationCell->True],
Cell[203603, 6802, 771, 15, 81, "Text"],
Cell[204377, 6819, 2953, 76, 253, "Input",
 InitializationCell->True],
Cell[207333, 6897, 196, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[207554, 6906, 718, 19, 92, "Input"],
Cell[208275, 6927, 28, 0, 28, "Output"]
}, Open  ]],
Cell[208318, 6930, 244, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[208587, 6941, 65, 1, 29, "Input"],
Cell[208655, 6944, 441, 15, 28, "Output"]
}, Open  ]],
Cell[209111, 6962, 341, 11, 35, "Text"],
Cell[209455, 6975, 342, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[209822, 6986, 91, 2, 28, "Input"],
Cell[209916, 6990, 201, 6, 28, "Output"]
}, Open  ]],
Cell[210132, 6999, 73, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[210230, 7003, 49, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[210304, 7008, 81, 1, 40, "Print",
 CellTags->"Info3580720268-4013843"],
Cell[210388, 7011, 1444, 45, 115, "Print",
 CellTags->"Info3580720268-4013843"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[211881, 7062, 49, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[211955, 7067, 81, 1, 40, "Print",
 CellTags->"Info3580720268-4013843"],
Cell[212039, 7070, 1444, 45, 115, "Print",
 CellTags->"Info3580720268-4013843"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[213532, 7121, 49, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[213606, 7126, 81, 1, 40, "Print",
 CellTags->"Info3580720268-4013843"],
Cell[213690, 7129, 1444, 45, 115, "Print",
 CellTags->"Info3580720268-4013843"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[215183, 7180, 49, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[215257, 7185, 81, 1, 40, "Print",
 CellTags->"Info3580720269-4013843"],
Cell[215341, 7188, 1444, 45, 115, "Print",
 CellTags->"Info3580720269-4013843"]
}, Open  ]]
}, Open  ]],
Cell[216812, 7237, 254, 6, 35, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[217115, 7249, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[217192, 7254, 105, 2, 20, "Item1Exercise"],
Cell[217300, 7258, 251, 6, 33, "Item1Exercise"],
Cell[217554, 7266, 337, 8, 33, "Item1Exercise"],
Cell[217894, 7276, 319, 7, 33, "Item1Exercise"],
Cell[218216, 7285, 239, 6, 33, "Item1Exercise"],
Cell[218458, 7293, 441, 15, 33, "Item1Exercise"],
Cell[218902, 7310, 218, 6, 33, "Item1Exercise"],
Cell[219123, 7318, 197, 6, 20, "Item1Exercise"],
Cell[219323, 7326, 272, 7, 33, "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
